@@ -1,3 +1,4 @@
+/* $Id: ftp.c,v 1.8 2001/11/29 10:22:58 ukai Exp $ */
 #include <stdio.h>
 #include <pwd.h>
 #include <Str.h>
@@ -31,7 +32,7 @@ typedef struct _FTP {
 
 typedef int STATUS;
 
-static FTP ftp;
+static FTP current_ftp;
 
 static Str
 read_response1(FTP ftp)
@@ -104,28 +105,29 @@ FtpLogin(FTP * ftp_return, char *host, char *user, char *pass)
     Str tmp;
     FTP ftp = New(struct _FTP);
     int fd;
-    *ftp_return = ftp;
+    *ftp_return = current_ftp = ftp;
     fd = openSocket(host, "ftp", 21);
     if (fd < 0)
 	return -1;
 #ifdef FTPPASS_HOSTNAMEGEN
-    if (!strcmp(user, "anonymous")) {
+    if (ftppass_hostnamegen && !strcmp(user, "anonymous")) {
 	size_t n = strlen(pass);
 
 	if (n > 0 && pass[n - 1] == '@') {
 	    struct sockaddr_in sockname;
 	    int socknamelen = sizeof(sockname);
 
-	    if (!getsockname(fd, (struct sockaddr *) &sockname, &socknamelen)) {
+	    if (!getsockname(fd, (struct sockaddr *)&sockname, &socknamelen)) {
 		struct hostent *sockent;
 		Str tmp2 = Strnew_charp(pass);
 
-		if (sockent = gethostbyaddr((char *) &sockname.sin_addr,
-					    sizeof(sockname.sin_addr),
-					    sockname.sin_family))
+		if ((sockent = gethostbyaddr((char *)&sockname.sin_addr,
+					     sizeof(sockname.sin_addr),
+					     sockname.sin_family)))
 		    Strcat_charp(tmp2, sockent->h_name);
 		else
-		    Strcat_m_charp(tmp2, "[", inet_ntoa(sockname.sin_addr), "]", NULL);
+		    Strcat_m_charp(tmp2, "[", inet_ntoa(sockname.sin_addr),
+				   "]", NULL);
 
 		pass = tmp2->ptr;
 	    }
@@ -139,7 +141,8 @@ FtpLogin(FTP * ftp_return, char *host, char *user, char *pass)
     if (atoi(tmp->ptr) != 220)
 	return -1;
     if (fmInitialized) {
-	message(Sprintf("Sending FTP username (%s) to remote server.", user)->ptr, 0, 0);
+	message(Sprintf("Sending FTP username (%s) to remote server.", user)->
+		ptr, 0, 0);
 	refresh();
     }
     tmp = Sprintf("USER %s\r\n", user);
@@ -188,18 +191,18 @@ ftp_pasv(FTP ftp)
     Str tmp;
     int family;
 #ifdef INET6
-    struct sockaddr_storage sin;
-    int sinlen, port;
+    struct sockaddr_storage sockaddr;
+    int sockaddrlen, port;
     unsigned char d1, d2, d3, d4;
     char abuf[INET6_ADDRSTRLEN];
 #endif
 
 #ifdef INET6
-    sinlen = sizeof(sin);
+    sockaddrlen = sizeof(sockaddr);
     if (getpeername(fileno(ftp->wcontrol),
-		    (struct sockaddr *)&sin, &sinlen) < 0)
+		    (struct sockaddr *)&sockaddr, &sockaddrlen) < 0)
 	return -1;
-    family = sin.ss_family;
+    family = sockaddr.ss_family;
 #else
     family = AF_INET;
 #endif
@@ -211,15 +214,14 @@ ftp_pasv(FTP ftp)
 	tmp = read_response(ftp);
 	if (atoi(tmp->ptr) != 229)
 	    return -1;
-	for (p = tmp->ptr + 4; *p && *p != '('; p++);
+	for (p = tmp->ptr + 4; *p && *p != '('; p++) ;
 	if (*p == '\0')
 	    return -1;
 	if (sscanf(++p, "%c%c%c%d%c", &d1, &d2, &d3, &port, &d4) != 5
 	    || d1 != d2 || d1 != d3 || d1 != d4)
 	    return -1;
-	if (getnameinfo((struct sockaddr *)&sin, sinlen,
-			abuf, sizeof(abuf),
-			NULL, 0, NI_NUMERICHOST) != 0)
+	if (getnameinfo((struct sockaddr *)&sockaddr, sockaddrlen,
+			abuf, sizeof(abuf), NULL, 0, NI_NUMERICHOST) != 0)
 	    return -1;
 	tmp = Sprintf("%s", abuf);
 	data_s = openSocket(tmp->ptr, "", port);
@@ -231,7 +233,7 @@ ftp_pasv(FTP ftp)
 	tmp = read_response(ftp);
 	if (atoi(tmp->ptr) != 227)
 	    return -1;
-	for (p = tmp->ptr + 4; *p && !IS_DIGIT(*p); p++);
+	for (p = tmp->ptr + 4; *p && !IS_DIGIT(*p); p++) ;
 	if (*p == '\0')
 	    return -1;
 	sscanf(p, "%d,%d,%d,%d,%d,%d", &n1, &n2, &n3, &n4, &p1, &p2);
@@ -272,7 +274,7 @@ FtpOpenReadBody(FTP ftp, char *path)
     fflush(ftp->wcontrol);
     tmp = read_response(ftp);
     if (tmp->ptr[0] == '5') {
-       return -1;
+	return -1;
     }
     return 0;
 }
@@ -281,7 +283,7 @@ int
 FtpOpenRead(FTP ftp, char *path)
 {
     if (ftp_pasv(ftp) < 0)
-       return -1;
+	return -1;
     if (FtpOpenReadBody(ftp, path) < 0) {
 	fclose(ftp->data);
 	ftp->data = NULL;
@@ -294,9 +296,9 @@ int
 Ftpfclose(FILE * f)
 {
     fclose(f);
-    if (f == ftp->data)
-	ftp->data = NULL;
-    read_response(ftp);
+    if (f == current_ftp->data)
+	current_ftp->data = NULL;
+    read_response(current_ftp);
     return 0;
 }
 
@@ -322,7 +324,7 @@ int
 FtpData(FTP ftp, char *cmd, char *arg, char *mode)
 {
     if (ftp_pasv(ftp) < 0)
-       return -1;
+	return -1;
 
     return FtpDataBody(ftp, cmd, arg, mode);
 }
@@ -376,8 +378,6 @@ Str FTPDIRtmp;
 
 static int ex_ftpdir_name_size_date(char *, char **, char **, char **);
 static int ftp_system(FTP);
-static char *ftp_escape_str(char *);
-static char *ftp_restore_str(char *);
 
 #define	SERVER_NONE	0
 #define	UNIXLIKE_SERVER	1
@@ -388,7 +388,7 @@ static char *ftp_restore_str(char *);
 #define	FTPDIR_FILE	3
 
 FILE *
-openFTP(ParsedURL * pu)
+openFTP(ParsedURL *pu)
 {
     Str tmp2 = Strnew();
     Str tmp3 = Strnew();
@@ -401,9 +401,9 @@ openFTP(ParsedURL * pu)
     char *qdir;
     char **flist;
     int i, nfile, nfile_max = 100;
-    Str pwd;
+    Str pwd = NULL;
     int add_auth_cookie_flag;
-    char *realpath = NULL;
+    char *realpathname = NULL;
 #ifdef JP_CHARSET
     char code = '\0', ic;
     Str pathStr;
@@ -429,7 +429,7 @@ openFTP(ParsedURL * pu)
 		term_cbreak();
 	    }
 	    else {
-		pwd = Strnew_charp((char *) getpass("Password: "));
+		pwd = Strnew_charp((char *)getpass("Password: "));
 	    }
 	    add_auth_cookie_flag = 1;
 	}
@@ -446,7 +446,7 @@ openFTP(ParsedURL * pu)
 	Strcat_char(tmp2, '@');
 	pass = tmp2->ptr;
     }
-    s = FtpLogin(&ftp, pu->host, user, pass);
+    s = FtpLogin(&current_ftp, pu->host, user, pass);
     if (FtpError(s))
 	return NULL;
     if (add_auth_cookie_flag)
@@ -454,69 +454,69 @@ openFTP(ParsedURL * pu)
     if (pu->file == NULL || *pu->file == '\0')
 	goto ftp_dir;
     else
-	realpath = file_unquote(pu->file);
+	realpathname = file_unquote(pu->file);
 
-    if (pu->file[strlen(pu->file)-1] == '/')
-       goto ftp_dir;
+    if (pu->file[strlen(pu->file) - 1] == '/')
+	goto ftp_dir;
 
     /* Get file */
-    FtpBinary(ftp);
-    if (ftp_pasv(ftp) < 0) {
-       FtpBye(ftp);
-       return NULL;
+    FtpBinary(current_ftp);
+    if (ftp_pasv(current_ftp) < 0) {
+	FtpBye(current_ftp);
+	return NULL;
     }
-    s = FtpOpenReadBody(ftp, realpath);
+    s = FtpOpenReadBody(current_ftp, realpathname);
     if (!FtpError(s)) {
 #ifdef JP_CHARSET
-	pathStr = Strnew_charp(realpath);
+	pathStr = Strnew_charp(realpathname);
 	if ((ic = checkShiftCode(pathStr, code)) != '\0') {
 	    pathStr = conv_str(pathStr, (code = ic), InnerCode);
-	    realpath = pathStr->ptr;
+	    realpathname = pathStr->ptr;
 	}
 #endif				/* JP_CHARSET */
-	pu->file = realpath;
-	return FTPDATA(ftp);
+	pu->file = realpathname;
+	return FTPDATA(current_ftp);
     }
     goto ftp_dir1;
 
     /* Get directory */
   ftp_dir:
-    if (ftp_pasv(ftp) < 0) {
-       FtpBye(ftp);
-       return NULL;
+    if (ftp_pasv(current_ftp) < 0) {
+	FtpBye(current_ftp);
+	return NULL;
     }
   ftp_dir1:
     pu->scheme = SCM_FTPDIR;
     FTPDIRtmp = Strnew();
-    sv_type = ftp_system(ftp);
+    sv_type = ftp_system(current_ftp);
     if (pu->file == NULL || *pu->file == '\0') {
 	if (sv_type == UNIXLIKE_SERVER) {
-           s = FtpDataBody(ftp, "LIST", NULL, "r");
+	    s = FtpDataBody(current_ftp, "LIST", NULL, "r");
 	}
 	else {
-           s = FtpDataBody(ftp, "NLST", NULL, "r");
+	    s = FtpDataBody(current_ftp, "NLST", NULL, "r");
 	}
 	curdir = Strnew_charp("/");
     }
     else {
 	if (sv_type == UNIXLIKE_SERVER) {
-	    s = FtpCwd(ftp, realpath);
+	    s = FtpCwd(current_ftp, realpathname);
 	    if (!FtpError(s)) {
-        s = FtpDataBody(ftp, "LIST", NULL, "r");
+		s = FtpDataBody(current_ftp, "LIST", NULL, "r");
 	    }
 	}
 	else {
-           s = FtpDataBody(ftp, "NLST %s", realpath, "r");
+	    s = FtpDataBody(current_ftp, "NLST %s", realpathname, "r");
 	}
-	if (realpath[0] == '/')
-	    curdir = Strnew_charp(realpath);
+	if (realpathname[0] == '/')
+	    curdir = Strnew_charp(realpathname);
 	else
-	    curdir = Sprintf("/%s", realpath);
+	    curdir = Sprintf("/%s", realpathname);
 	if (Strlastchar(curdir) != '/')
 	    Strcat_char(curdir, '/');
     }
     if (FtpError(s)) {
-	FtpBye(ftp);
+	FtpBye(current_ftp);
 	return NULL;
     }
     host = Strnew_charp("ftp://");
@@ -527,8 +527,10 @@ openFTP(ParsedURL * pu)
     if (Strlastchar(host) == '/')
 	Strshrink(host, 1);
     qdir = html_quote(curdir->ptr);
-    FTPDIRtmp = Sprintf("<html><head><title>%s%s</title></head><body><h1>Index of %s%s</h1>\n",
-			host->ptr, qdir, host->ptr, qdir);
+    FTPDIRtmp =
+	Sprintf
+	("<html><head><title>%s%s</title></head><body><h1>Index of %s%s</h1>\n",
+	 host->ptr, qdir, host->ptr, qdir);
     curdir = Strnew_charp(file_quote(curdir->ptr));
     qdir = curdir->ptr;
     tmp2 = Strdup(curdir);
@@ -544,20 +546,20 @@ openFTP(ParsedURL * pu)
 	Strcat_charp(FTPDIRtmp, "<ul><li><a href=\"");
     }
     Strcat_m_charp(FTPDIRtmp, host->ptr,
-		   html_quote(tmp2->ptr),
-		   "\">[Upper Directory]</a>\n", NULL);
+		   html_quote(tmp2->ptr), "\">[Upper Directory]</a>\n", NULL);
 
     flist = New_N(char *, nfile_max);
     nfile = 0;
     if (sv_type == UNIXLIKE_SERVER) {
 	char *name, *date, *size, *type_str;
-	int ftype, max_len, len, i, j;
+	int ftype, max_len, len, j;
 	Str line_tmp;
 
 	max_len = 0;
-	while (tmp2 = Strfgets(FTPDATA(ftp)), tmp2->length > 0) {
+	while (tmp2 = Strfgets(FTPDATA(current_ftp)), tmp2->length > 0) {
 	    Strchop(tmp2);
-	    if ((ftype = ex_ftpdir_name_size_date(tmp2->ptr, &name, &date, &size))
+	    if ((ftype =
+		 ex_ftpdir_name_size_date(tmp2->ptr, &name, &date, &size))
 		== FTPDIR_NONE) {
 		continue;
 	    }
@@ -580,7 +582,8 @@ openFTP(ParsedURL * pu)
 	    }
 	    if (max_len < len)
 		max_len = len;
-	    line_tmp = Sprintf("%s%s %-12.12s %6.6s", name, type_str, date, size);
+	    line_tmp =
+		Sprintf("%s%s %-12.12s %6.6s", name, type_str, date, size);
 	    flist[nfile++] = line_tmp->ptr;
 	    if (nfile == nfile_max) {
 		nfile_max *= 2;
@@ -609,8 +612,7 @@ openFTP(ParsedURL * pu)
 			   host->ptr,
 			   qdir,
 			   html_quote(file_quote(fn)),
-			   "\">",
-			   html_quote(fn), NULL);
+			   "\">", html_quote(fn), NULL);
 	    if (ftype == FTPDIR_DIR) {
 		Strcat_charp(FTPDIRtmp, "/");
 		len++;
@@ -633,7 +635,7 @@ openFTP(ParsedURL * pu)
 	Strcat_charp(FTPDIRtmp, "</pre></body></html>\n");
     }
     else {
-	while (tmp2 = Strfgets(FTPDATA(ftp)), tmp2->length > 0) {
+	while (tmp2 = Strfgets(FTPDATA(current_ftp)), tmp2->length > 0) {
 	    Strchop(tmp2);
 	    flist[nfile++] = mybasename(tmp2->ptr);
 	    if (nfile == nfile_max) {
@@ -647,15 +649,13 @@ openFTP(ParsedURL * pu)
 	    Strcat_m_charp(FTPDIRtmp, "<li><a href=\"",
 			   host->ptr, qdir,
 			   html_quote(file_quote(fn)),
-			   "\">",
-			   html_quote(fn),
-			   "</a>\n", NULL);
+			   "\">", html_quote(fn), "</a>\n", NULL);
 	}
 	Strcat_charp(FTPDIRtmp, "</ul></body></html>\n");
     }
 
-    FtpClose(ftp);
-    FtpBye(ftp);
+    FtpClose(current_ftp);
+    FtpBye(current_ftp);
     return NULL;
 }
 
@@ -668,37 +668,13 @@ ftp_system(FTP ftp)
     fwrite("SYST\r\n", 6, sizeof(char), ftp->wcontrol);
     fflush(ftp->wcontrol);
     tmp = read_response(ftp);
-    if (strstr(tmp->ptr, "UNIX") != NULL
-	|| !strncmp(tmp->ptr + 4, "Windows_NT", 10)) {	/* :-) */
+    if (strstr(tmp->ptr, "UNIX") != NULL || !strncmp(tmp->ptr + 4, "Windows_NT", 10)) {	/* :-) */
 	sv_type = UNIXLIKE_SERVER;
     }
 
     return (sv_type);
 }
 
-static char *
-ftp_escape_str(char *str)
-{
-    Str s = Strnew();
-    char *p, buf[5];
-    unsigned char c;
-
-    for (; (c = (unsigned char) *str) != '\0'; str++) {
-	p = NULL;
-	if (c < '!' || c > '~'
-	    || c == '#' || c == '?' || c == '+'
-	    || c == '&' || c == '<' || c == '>' || c == '"' || c == '%') {
-	    sprintf(buf, "%%%02X", c);
-	    p = buf;
-	}
-	if (p)
-	    Strcat_charp(s, p);
-	else
-	    Strcat_char(s, *str);
-    }
-    return s->ptr;
-}
-
 #define XD_CTOD(c) {\
   if (c >= '0' && c <= '9') {\
     c -= (unsigned char)'0';\
@@ -711,35 +687,6 @@ ftp_escape_str(char *str)
   }\
 }
 
-static char *
-ftp_restore_str(char *str)
-{
-    Str s = Strnew();
-    char *p;
-    unsigned char c, code[2];
-
-    for (; *str; str++) {
-	p = NULL;
-	if (*str == '%' && str[1] != '\0' && str[2] != '\0') {
-	    c = (unsigned char) str[1];
-	    XD_CTOD(c)
-		code[0] = c * (unsigned char) 16;
-	    c = (unsigned char) str[2];
-	    XD_CTOD(c)
-		code[0] += c;
-	    code[1] = '\0';
-	    p = (char *) code;
-	    str += 2;
-	}
-      skip:
-	if (p)
-	    Strcat_charp(s, p);
-	else
-	    Strcat_char(s, *str);
-    }
-    return s->ptr;
-}
-
 #define EX_SKIP_SPACE(cp) {\
   while (IS_SPACE(*cp) && *cp != '\0') cp++;\
   if (*cp == '\0') {\
@@ -854,7 +801,7 @@ size_int2str(unsigned long size)
     double dtmp;
     char *size_format, *unit_str;
 
-    dtmp = (double) size;
+    dtmp = (double)size;
     for (unit = 0; unit < 3; unit++) {
 	if (dtmp < 1024) {
 	    break;
@@ -894,8 +841,8 @@ closeFTP(FILE * f)
 {
     if (f) {
 	fclose(f);
-	if (f == ftp->data)
-	    ftp->data = NULL;
+	if (f == current_ftp->data)
+	    current_ftp->data = NULL;
     }
-    FtpBye(ftp);
+    FtpBye(current_ftp);
 }
