@@ -1,13 +1,10 @@
-/* $Id: indep.c,v 1.3 2001/11/15 00:32:13 a-ito Exp $ */
+/* $Id: indep.c,v 1.18 2001/12/14 17:10:54 ukai Exp $ */
 #include "fm.h"
 #include <stdio.h>
 #include <pwd.h>
 #include <sys/param.h>
 #include <sys/types.h>
 #include <stdlib.h>
-#ifdef __EMX__
-#include <strings.h>        /* for bcopy() */
-#endif                /* __EMX__ */
 #include "indep.h"
 #include "Str.h"
 #include "gc.h"
@@ -21,7 +18,7 @@ allocStr(const char *s, int len)
 
     if (s == NULL)
 	return NULL;
-    if (len == 0)
+    if (len < 0)
 	len = strlen(s);
     ptr = NewAtom_N(char, len + 1);
     if (ptr == NULL) {
@@ -36,28 +33,25 @@ allocStr(const char *s, int len)
 int
 strCmp(const void *s1, const void *s2)
 {
-    unsigned char *p1 = *(unsigned char **) s1;
-    unsigned char *p2 = *(unsigned char **) s2;
-
-    while ((*p1 != '\0') && (*p1 == *p2)) {
-	p1++;
-	p2++;
-    }
-    return (*p1 - *p2);
+    return strcmp(*(const char **)s1, *(const char **)s2);
 }
 
 char *
 currentdir()
 {
     char *path;
-#ifdef GETCWD
+#ifdef HAVE_GETCWD
+#ifdef MAXPATHLEN
     path = NewAtom_N(char, MAXPATHLEN);
     getcwd(path, MAXPATHLEN);
-#else				/* not GETCWD */
-#ifdef GETWD
+#else
+    path = getcwd(NULL, 0);
+#endif
+#else				/* not HAVE_GETCWD */
+#ifdef HAVE_GETWD
     path = NewAtom_N(char, 1024);
     getwd(path);
-#else				/* not GETWD */
+#else				/* not HAVE_GETWD */
     FILE *f;
     char *p;
     path = NewAtom_N(char, 1024);
@@ -69,8 +63,8 @@ currentdir()
 	    *p = '\0';
 	    break;
 	}
-#endif				/* not GETWD */
-#endif				/* not GETCWD */
+#endif				/* not HAVE_GETWD */
+#endif				/* not HAVE_GETCWD */
     return path;
 }
 
@@ -79,7 +73,7 @@ cleanupName(char *name)
 {
     char *buf, *p, *q;
 
-    buf = allocStr(name, 0);
+    buf = allocStr(name, -1);
     p = buf;
     q = name;
     while (*q != '\0') {
@@ -95,7 +89,7 @@ cleanupName(char *name)
 		q += 3;
 	    }
 	    else {
-		while (p != buf && *--p != '/');	/* ->foo/FOO */
+		while (p != buf && *--p != '/') ;	/* ->foo/FOO */
 		*p = '\0';
 		q += 3;
 		strcat(buf, q);
@@ -109,7 +103,7 @@ cleanupName(char *name)
 		/* ../../..     */
 	    }
 	    else {
-		while (p != buf && *--p != '/');	/* ->foo/ */
+		while (p != buf && *--p != '/') ;	/* ->foo/ */
 		*++p = '\0';
 	    }
 	    break;
@@ -125,13 +119,6 @@ cleanupName(char *name)
 	}
 	else if (strncmp(p, "//", 2) == 0) {	/* foo//bar */
 	    /* -> foo/bar           */
-#if 0 /* ifdef SUPPORT_NETBIOS_SHARE */
-           if (p == buf) {       /* //DRIVE/foo or //host/path */
-		p += 2;
-		q += 2;
-		continue;
-	    }
-#endif                /* SUPPORT_NETBIOS_SHARE */
 	    *p = '\0';
 	    q++;
 	    strcat(buf, q);
@@ -179,21 +166,27 @@ expandPath(char *name)
     return extpath->ptr;
 }
 
+#ifndef HAVE_STRCASESTR
 /* string search using the simplest algorithm */
 char *
-strcasestr(char *s1, char *s2)
+strcasestr(const char *s1, const char *s2)
 {
     int len1, len2;
+    if (s2 == NULL)
+	return (char *)s1;
+    if (*s2 == '\0')
+	return (char *)s1;
     len1 = strlen(s1);
     len2 = strlen(s2);
     while (*s1 && len1 >= len2) {
 	if (strncasecmp(s1, s2, len2) == 0)
-	    return s1;
+	    return (char *)s1;
 	s1++;
 	len1--;
     }
     return 0;
 }
+#endif
 
 static int
 strcasematch(char *s1, char *s2)
@@ -234,12 +227,9 @@ remove_space(char *str)
 {
     char *p, *q;
 
-    for (p = str; *p && IS_SPACE(*p); p++)
-	;
-    for (q = p; *q; q++)
-	;
-    for (; q > p && IS_SPACE(*(q-1)); q--)
-	;
+    for (p = str; *p && IS_SPACE(*p); p++) ;
+    for (q = p; *q; q++) ;
+    for (; q > p && IS_SPACE(*(q - 1)); q--) ;
     if (*q != '\0')
 	return Strnew_charp_n(p, q - p)->ptr;
     return p;
@@ -262,8 +252,7 @@ void
 cleanup_line(Str s, int mode)
 {
     if (s->length >= 2 &&
-	s->ptr[s->length - 2] == '\r' &&
-	s->ptr[s->length - 1] == '\n') {
+	s->ptr[s->length - 2] == '\r' && s->ptr[s->length - 1] == '\n') {
 	Strshrink(s, 2);
 	Strcat_char(s, '\n');
     }
@@ -283,8 +272,9 @@ cleanup_line(Str s, int mode)
 int
 getescapechar(char **str)
 {
-    int i, dummy = -1;
+    int dummy = -1;
     char *p = *str, *q;
+    int strict_entity = TRUE;
 
     if (*p == '&')
 	p++;
@@ -292,45 +282,54 @@ getescapechar(char **str)
 	p++;
 	if (*p == 'x' || *p == 'X') {
 	    p++;
-	    if (! IS_XDIGIT(*p)) {
+	    if (!IS_XDIGIT(*p)) {
 		*str = p;
 		return -1;
 	    }
-	    q = p;
-	    for (p++; IS_XDIGIT(*p); p++)
-		;
-	    q = allocStr(q, p - q);
+	    for (dummy = GET_MYCDIGIT(*p), p++; IS_XDIGIT(*p); p++)
+		dummy = dummy * 0x10 + GET_MYCDIGIT(*p);
 	    if (*p == ';')
 		p++;
 	    *str = p;
-	    sscanf(q, "%x", &dummy);
 	    return dummy;
-	} else {
-	    if (! IS_DIGIT(*p)) {
+	}
+	else {
+	    if (!IS_DIGIT(*p)) {
 		*str = p;
 		return -1;
 	    }
-	    q = p;
-	    for (p++; IS_DIGIT(*p); p++)
-		;
-	    q = allocStr(q, p - q);
+	    for (dummy = GET_MYCDIGIT(*p), p++; IS_DIGIT(*p); p++)
+		dummy = dummy * 10 + GET_MYCDIGIT(*p);
 	    if (*p == ';')
 		p++;
 	    *str = p;
-	    sscanf(q, "%d", &dummy);
 	    return dummy;
 	}
     }
-    if (! IS_ALPHA(*p)) {
+    if (!IS_ALPHA(*p)) {
 	*str = p;
 	return -1;
     }
     q = p;
-    for (p++; IS_ALNUM(*p); p++)
-	;
+    for (p++; IS_ALNUM(*p); p++) ;
     q = allocStr(q, p - q);
+    if (strcasestr("lt gt amp quot nbsp", q) && *p != '=') {
+	/* a character entity MUST be terminated with ";". However,
+	 * there's MANY web pages which uses &lt , &gt or something
+	 * like them as &lt;, &gt;, etc. Therefore, we treat the most
+	 * popular character entities (including &#xxxx;) without
+	 * the last ";" as character entities. If the trailing character
+	 * is "=", it must be a part of query in an URL. So &lt=, &gt=, etc.
+	 * are not regarded as character entities.
+	 */
+	strict_entity = FALSE;
+    }
     if (*p == ';')
 	p++;
+    else if (strict_entity) {
+	*str = p;
+	return -1;
+    }
     *str = p;
     return getHash_si(&entity, q, -1);
 }
@@ -398,7 +397,7 @@ html_unquote(char *str)
     Str tmp = NULL;
     char *p, *q;
 
-    for (p = str; *p; ) {
+    for (p = str; *p;) {
 	if (*p == '&') {
 	    if (tmp == NULL)
 		tmp = Strnew_charp_n(str, (int)(p - str));
@@ -417,45 +416,30 @@ html_unquote(char *str)
     return str;
 }
 
-static int
-url_unquote_char(char **str)
-{
-    char *p = *str;
-    char buf[3];
-    int n;
+static char xdigit[0x10] = "0123456789ABCDEF";
 
-    if (*p != '%')
-	return -1;
-    p++;
-    if (IS_XDIGIT(*p)) {
-	buf[0] = *(p++);
-	if (IS_XDIGIT(*p)) {
-	    buf[1] = *(p++);
-	    buf[2] = '\0';
-	} else
-	    buf[1] = '\0';
-	if (sscanf(buf, "%x", &n)) {
-	    *str = p;
-	    return n;
-	}
-    }
-    return -1;
-}
+#define url_unquote_char(pstr) \
+  (IS_XDIGIT((*(pstr))[1]) ? \
+    (IS_XDIGIT((*(pstr))[2]) ? \
+    (*(pstr) += 3, (GET_MYCDIGIT((*(pstr))[-2]) << 4) | GET_MYCDIGIT((*(pstr))[-1])) : \
+    (*(pstr) += 2, GET_MYCDIGIT((*(pstr))[-1]))) : \
+   -1)
 
 char *
 url_quote(char *str)
 {
     Str tmp = NULL;
     char *p;
-    char buf[4];
 
     for (p = str; *p; p++) {
-	if (IS_CNTRL(*p) || *p == ' ' || ! IS_ASCII(*p)) {
+	if (IS_CNTRL(*p) || *p == ' ' || !IS_ASCII(*p)) {
 	    if (tmp == NULL)
 		tmp = Strnew_charp_n(str, (int)(p - str));
-	    sprintf(buf, "%%%02X", (unsigned char)*p);
-	    Strcat_charp(tmp, buf);
-	} else {
+	    Strcat_char(tmp, '%');
+	    Strcat_char(tmp, xdigit[((unsigned char)*p >> 4) & 0xF]);
+	    Strcat_char(tmp, xdigit[(unsigned char)*p & 0xF]);
+	}
+	else {
 	    if (tmp)
 		Strcat_char(tmp, *p);
 	}
@@ -472,11 +456,11 @@ url_unquote(char *str)
     char *p, *q;
     int c;
 
-    for (p = str; *p; ) {
+    for (p = str; *p;) {
 	if (*p == '%') {
 	    q = p;
 	    c = url_unquote_char(&q);
-	    if (c >= 0 && (IS_CNTRL(c) || c == ' ' || ! IS_ASCII(c))) {
+	    if (c >= 0 && (IS_CNTRL(c) || c == ' ' || !IS_ASCII(c))) {
 		if (tmp == NULL)
 		    tmp = Strnew_charp_n(str, (int)(p - str));
 		if (c != '\0' && c != '\n' && c != '\r')
@@ -502,13 +486,14 @@ file_quote(char *str)
     char buf[4];
 
     for (p = str; *p; p++) {
-	if (IS_CNTRL(*p) || *p == ' ' || ! IS_ASCII(*p) || *p == '+' ||
+	if (IS_CNTRL(*p) || *p == ' ' || !IS_ASCII(*p) || *p == '+' ||
 	    *p == ':' || *p == '#' || *p == '?' || *p == '&' || *p == '%') {
 	    if (tmp == NULL)
 		tmp = Strnew_charp_n(str, (int)(p - str));
 	    sprintf(buf, "%%%02X", (unsigned char)*p);
 	    Strcat_charp(tmp, buf);
-	} else {
+	}
+	else {
 	    if (tmp)
 		Strcat_char(tmp, *p);
 	}
@@ -525,7 +510,7 @@ file_unquote(char *str)
     char *p, *q;
     int c;
 
-    for (p = str; *p; ) {
+    for (p = str; *p;) {
 	if (*p == '%') {
 	    q = p;
 	    c = url_unquote_char(&q);
@@ -575,12 +560,14 @@ Str_form_quote(Str x)
 	    if (tmp == NULL)
 		tmp = Strnew_charp_n(x->ptr, (int)(p - x->ptr));
 	    Strcat_char(tmp, '+');
-	} else if (! is_url_safe(*p)) {
+	}
+	else if (!is_url_safe(*p)) {
 	    if (tmp == NULL)
 		tmp = Strnew_charp_n(x->ptr, (int)(p - x->ptr));
 	    sprintf(buf, "%%%02X", (unsigned char)*p);
 	    Strcat_charp(tmp, buf);
-	} else {
+	}
+	else {
 	    if (tmp)
 		Strcat_char(tmp, *p);
 	}
@@ -597,14 +584,15 @@ Str_form_unquote(Str x)
     char *p = x->ptr, *ep = x->ptr + x->length, *q;
     int c;
 
-    for (; p < ep; ) {
+    for (; p < ep;) {
 	if (*p == '+') {
 	    if (tmp == NULL)
 		tmp = Strnew_charp_n(x->ptr, (int)(p - x->ptr));
 	    Strcat_char(tmp, ' ');
 	    p++;
 	    continue;
-	} else if (*p == '%') {
+	}
+	else if (*p == '%') {
 	    q = p;
 	    c = url_unquote_char(&q);
 	    if (c >= 0) {
@@ -646,12 +634,13 @@ shell_quote(char *str)
     char *p;
 
     for (p = str; *p; p++) {
-        if (! is_shell_safe(*p)) {
+	if (!is_shell_safe(*p)) {
 	    if (tmp == NULL)
 		tmp = Strnew_charp_n(str, (int)(p - str));
-            Strcat_char(tmp, '\\');
+	    Strcat_char(tmp, '\\');
 	    Strcat_char(tmp, *p);
-	} else {
+	}
+	else {
 	    if (tmp)
 		Strcat_char(tmp, *p);
 	}
@@ -661,6 +650,34 @@ shell_quote(char *str)
     return str;
 }
 
+static char *
+w3m_dir(const char *name, char *dft)
+{
+#ifdef USE_PATH_ENVVAR
+    char *value = getenv(name);
+    return value ? value : dft;
+#else
+    return dft;
+#endif
+}
+
+char *
+w3m_lib_dir()
+{
+    return w3m_dir("W3M_LIB_DIR", LIB_DIR);
+}
+
+char *
+w3m_etc_dir()
+{
+    return w3m_dir("W3M_ETC_DIR", ETC_DIR);
+}
+
+char *
+w3m_help_dir()
+{
+    return w3m_dir("W3M_HELP_DIR", HELP_DIR);
+}
 /* Local Variables:    */
 /* c-basic-offset: 4   */
 /* tab-width: 8        */
