@@ -1,4 +1,4 @@
-/* $Id: mailcap.c,v 1.3.2.4 2000/12/02 11:33:11 okabe Exp $ */
+/* $Id: mailcap.c,v 1.9 2001/11/30 10:00:06 ukai Exp $ */
 #include "fm.h"
 #include "myctype.h"
 #include <stdio.h>
@@ -6,29 +6,14 @@
 #include "parsetag.h"
 #include "local.h"
 
-static struct mailcap DefaultMailcap[] =
-{
+static struct mailcap DefaultMailcap[] = {
     {"image/*", "xv %s", 0, NULL, NULL, NULL},	/* */
     {"audio/basic", "showaudio %s", 0, NULL, NULL, NULL},
     {NULL, NULL, 0, NULL, NULL, NULL}
 };
 
-void
-initMailcap()
-{
-    int i;
-    TextListItem *tl;
-
-    if (non_null(mailcap_files))
-	mailcap_list = make_domain_list(mailcap_files);
-    else
-	mailcap_list = NULL;
-    if (mailcap_list == NULL)
-	return;
-    UserMailcap = New_N(struct mailcap *, mailcap_list->nitem);
-    for (i = 0, tl = mailcap_list->first; tl; i++, tl = tl->next)
-	UserMailcap[i] = loadMailcap(tl->ptr);
-}
+static TextList *mailcap_list;
+static struct mailcap **UserMailcap;
 
 int
 mailcapMatch(struct mailcap *mcap, char *type)
@@ -74,7 +59,8 @@ searchMailcap(struct mailcap *table, char *type)
 	i = mailcapMatch(table, type);
 	if (i > level) {
 	    if (table->test) {
-		Str command = unquote_mailcap(table->test, type, NULL, NULL);
+		Str command =
+		    unquote_mailcap(table->test, type, NULL, NULL, NULL);
 		if (system(command->ptr) != 0)
 		    continue;
 	    }
@@ -86,7 +72,7 @@ searchMailcap(struct mailcap *table, char *type)
 }
 
 static int
-matchattr(char *p, char *attr, int len, Str * value)
+matchMailcapAttr(char *p, char *attr, int len, Str *value)
 {
     int quoted;
     char *q = NULL;
@@ -124,7 +110,7 @@ matchattr(char *p, char *attr, int len, Str * value)
     return 0;
 }
 
-int
+static int
 extractMailcapEntry(char *mcap_entry, struct mailcap *mcap)
 {
     int j, k;
@@ -140,7 +126,7 @@ extractMailcapEntry(char *mcap_entry, struct mailcap *mcap)
 	if (!IS_SPACE(p[j]))
 	    k = j;
     }
-    mcap->type = allocStr(p, (k >= 0)? k + 1 : j);
+    mcap->type = allocStr(p, (k >= 0) ? k + 1 : j);
     if (!p[j])
 	return 0;
     p += j + 1;
@@ -156,28 +142,29 @@ extractMailcapEntry(char *mcap_entry, struct mailcap *mcap)
 	else if (p[j] == '\\')
 	    quoted = 1;
     }
-    mcap->viewer = allocStr(p, (k >= 0)? k + 1 : j);
+    mcap->viewer = allocStr(p, (k >= 0) ? k + 1 : j);
     p += j;
 
     while (*p == ';') {
 	p++;
 	SKIP_BLANKS(p);
-	if (matchattr(p, "needsterminal", 13, NULL)) {
+	if (matchMailcapAttr(p, "needsterminal", 13, NULL)) {
 	    mcap->flags |= MAILCAP_NEEDSTERMINAL;
 	}
-	else if (matchattr(p, "copiousoutput", 13, NULL)) {
+	else if (matchMailcapAttr(p, "copiousoutput", 13, NULL)) {
 	    mcap->flags |= MAILCAP_COPIOUSOUTPUT;
 	}
-	else if (matchattr(p, "htmloutput", 10, NULL)) {
+	else if (matchMailcapAttr(p, "x-htmloutput", 12, NULL) ||
+		 matchMailcapAttr(p, "htmloutput", 10, NULL)) {
 	    mcap->flags |= MAILCAP_HTMLOUTPUT;
 	}
-	else if (matchattr(p, "test", 4, &tmp)) {
+	else if (matchMailcapAttr(p, "test", 4, &tmp)) {
 	    mcap->test = allocStr(tmp->ptr, tmp->length);
 	}
-	else if (matchattr(p, "nametemplate", 12, &tmp)) {
+	else if (matchMailcapAttr(p, "nametemplate", 12, &tmp)) {
 	    mcap->nametemplate = allocStr(tmp->ptr, tmp->length);
 	}
-	else if (matchattr(p, "edit", 4, &tmp)) {
+	else if (matchMailcapAttr(p, "edit", 4, &tmp)) {
 	    mcap->edit = allocStr(tmp->ptr, tmp->length);
 	}
 	quoted = 0;
@@ -192,7 +179,7 @@ extractMailcapEntry(char *mcap_entry, struct mailcap *mcap)
     return 1;
 }
 
-struct mailcap *
+static struct mailcap *
 loadMailcap(char *filename)
 {
     FILE *f;
@@ -232,6 +219,70 @@ loadMailcap(char *filename)
     return mcap;
 }
 
+void
+initMailcap()
+{
+    TextListItem *tl;
+    int i;
+
+    if (non_null(mailcap_files))
+	mailcap_list = make_domain_list(mailcap_files);
+    else
+	mailcap_list = NULL;
+    if (mailcap_list == NULL)
+	return;
+    UserMailcap = New_N(struct mailcap *, mailcap_list->nitem);
+    for (i = 0, tl = mailcap_list->first; tl; i++, tl = tl->next)
+	UserMailcap[i] = loadMailcap(tl->ptr);
+
+}
+
+char *
+acceptableMimeTypes()
+{
+    static Str types = NULL;
+    TextList *l;
+    Hash_si *mhash;
+    char *p;
+    int i;
+
+    if (types != NULL)
+	return types->ptr;
+
+    /* generate acceptable media types */
+    l = newTextList();
+    mhash = newHash_si(16);	/* XXX */
+    pushText(l, "text");
+    putHash_si(mhash, "text", 1);
+    pushText(l, "image");
+    putHash_si(mhash, "image", 1);
+    for (i = 0; i < mailcap_list->nitem; i++) {
+	struct mailcap *mp = UserMailcap[i];
+	char *mt;
+	if (mp == NULL)
+	    continue;
+	for (; mp->type; mp++) {
+	    p = strchr(mp->type, '/');
+	    if (p == NULL)
+		continue;
+	    mt = allocStr(mp->type, p - mp->type);
+	    if (getHash_si(mhash, mt, 0) == 0) {
+		pushText(l, mt);
+		putHash_si(mhash, mt, 1);
+	    }
+	}
+    }
+    while ((p = popText(l)) != NULL) {
+	if (types == NULL)
+	    types = Strnew();
+	else
+	    Strcat_charp(types, ", ");
+	Strcat_charp(types, p);
+	Strcat_charp(types, "/*");
+    }
+    return types->ptr;
+}
+
 struct mailcap *
 searchExtViewer(char *type)
 {
@@ -240,13 +291,13 @@ searchExtViewer(char *type)
 
     if (mailcap_list == NULL)
 	goto no_user_mailcap;
-    
+
     for (i = 0; i < mailcap_list->nitem; i++) {
 	if ((p = searchMailcap(UserMailcap[i], type)) != NULL)
 	    return p;
     }
 
- no_user_mailcap:
+  no_user_mailcap:
     return searchMailcap(DefaultMailcap, type);
 }
 
@@ -255,26 +306,77 @@ searchExtViewer(char *type)
 #define MC_PREC2  2
 #define MC_QUOTED 3
 
+#define MCF_SQUOTED (1 << 0)
+#define MCF_DQUOTED (1 << 1)
+
 Str
-unquote_mailcap(char *qstr, char *type, char *name, int *stat)
+quote_mailcap(char *s, int flag)
 {
-    Str str = Strnew();
+    Str d;
+
+    d = Strnew();
+
+    for (;; ++s)
+	switch (*s) {
+	case '\0':
+	    goto end;
+	case '$':
+	case '`':
+	case '"':
+	case '\\':
+	    if (!(flag & MCF_SQUOTED))
+		Strcat_char(d, '\\');
+
+	    Strcat_char(d, *s);
+	    break;
+	case '\'':
+	    if (flag & MCF_SQUOTED) {
+		Strcat_charp(d, "'\\''");
+		break;
+	    }
+	default:
+	    if (!flag && !IS_ALNUM(*s))
+		Strcat_char(d, '\\');
+	case '_':
+	case '.':
+	case ':':
+	case '/':
+	    Strcat_char(d, *s);
+	    break;
+	}
+  end:
+    return d;
+}
+
+
+static Str
+unquote_mailcap_loop(char *qstr, char *type, char *name, char *attr,
+		     int *mc_stat, int flag0)
+{
+    Str str, tmp, test, then;
     char *p;
-    int status = MC_NORMAL;
+    int status = MC_NORMAL, prev_status = MC_NORMAL, sp = 0, flag;
+
+    if (mc_stat)
+	*mc_stat = 0;
 
-    if (stat)
-	*stat = 0;
-    
     if (qstr == NULL)
 	return NULL;
 
-    for (p = qstr; *p; p++) {
+    str = Strnew();
+    tmp = test = then = NULL;
+
+    for (flag = flag0, p = qstr; *p; p++) {
 	if (status == MC_QUOTED) {
-	    Strcat_char(str, *p);
-	    status = MC_NORMAL;
+	    if (prev_status == MC_PREC2)
+		Strcat_char(tmp, *p);
+	    else
+		Strcat_char(str, *p);
+	    status = prev_status;
 	    continue;
 	}
 	else if (*p == '\\') {
+	    prev_status = status;
 	    status = MC_QUOTED;
 	    continue;
 	}
@@ -283,24 +385,37 @@ unquote_mailcap(char *qstr, char *type, char *name, int *stat)
 	    if (*p == '%') {
 		status = MC_PREC;
 	    }
-	    else
+	    else {
+		if (*p == '\'') {
+		    if (!flag0 && flag & MCF_SQUOTED)
+			flag &= ~MCF_SQUOTED;
+		    else if (!flag)
+			flag |= MCF_SQUOTED;
+		}
+		else if (*p == '"') {
+		    if (!flag0 && flag & MCF_DQUOTED)
+			flag &= ~MCF_DQUOTED;
+		    else if (!flag)
+			flag |= MCF_DQUOTED;
+		}
 		Strcat_char(str, *p);
+	    }
 	    break;
 	case MC_PREC:
 	    if (IS_ALPHA(*p)) {
 		switch (*p) {
 		case 's':
 		    if (name) {
-			Strcat_charp(str, name);
-			if (stat)
-			    *stat |= MCSTAT_REPNAME;
+			Strcat_charp(str, quote_mailcap(name, flag)->ptr);
+			if (mc_stat)
+			    *mc_stat |= MCSTAT_REPNAME;
 		    }
 		    break;
 		case 't':
 		    if (type) {
-			Strcat_charp(str, type);
-			if (stat)
-			    *stat |= MCSTAT_REPTYPE;
+			Strcat_charp(str, quote_mailcap(type, flag)->ptr);
+			if (mc_stat)
+			    *mc_stat |= MCSTAT_REPTYPE;
 		    }
 		    break;
 		}
@@ -308,17 +423,50 @@ unquote_mailcap(char *qstr, char *type, char *name, int *stat)
 	    }
 	    else if (*p == '{') {
 		status = MC_PREC2;
+		test = then = NULL;
+		tmp = Strnew();
 	    }
 	    else if (*p == '%') {
 		Strcat_char(str, *p);
 	    }
 	    break;
 	case MC_PREC2:
-	    if (*p == '}') {
+	    if (sp > 0 || *p == '{') {
+		Strcat_char(tmp, *p);
+
+		switch (*p) {
+		case '{':
+		    ++sp;
+		    break;
+		case '}':
+		    --sp;
+		    break;
+		default:
+		    break;
+		}
+	    }
+	    else if (*p == '}') {
+		char *q;
+		if (attr && (q = strcasestr(attr, tmp->ptr)) != NULL &&
+		    (q == attr || IS_SPACE(*(q - 1)) || *(q - 1) == ';') &&
+		    matchattr(q, tmp->ptr, tmp->length, &tmp)) {
+		    Strcat_charp(str, quote_mailcap(tmp->ptr, flag)->ptr);
+		    if (mc_stat)
+			*mc_stat |= MCSTAT_REPPARAM;
+		}
 		status = MC_NORMAL;
 	    }
+	    else {
+		Strcat_char(tmp, *p);
+	    }
 	    break;
 	}
     }
     return str;
 }
+
+Str
+unquote_mailcap(char *qstr, char *type, char *name, char *attr, int *mc_stat)
+{
+    return unquote_mailcap_loop(qstr, type, name, attr, mc_stat, 0);
+}
