@@ -1,4 +1,4 @@
-
+/* $Id: menu.c,v 1.9 2001/12/10 17:02:44 ukai Exp $ */
 /* 
  * w3m menu.c
  */
@@ -10,7 +10,7 @@
 #include "myctype.h"
 #include "regex.h"
 
-#ifdef MOUSE
+#ifdef USE_MOUSE
 #ifdef USE_GPM
 #include <gpm.h>
 static int gpm_process_menu_mouse(Gpm_Event * event, void *data);
@@ -26,15 +26,13 @@ extern int sysm_process_mouse(int, int, int, int);
 static int X_Mouse_Selection;
 extern int do_getch();
 #define getch()	do_getch()
-#endif				/* defined(USE_GPM) || * * * * * *
-				 * defined(USE_SYSMOUSE) */
-#endif				/* MOUSE */
+#endif				/* defined(USE_GPM) || defined(USE_SYSMOUSE) */
+#endif				/* USE_MOUSE */
 
-#ifdef MENU
+#ifdef USE_MENU
 
 #ifdef KANJI_SYMBOLS
-static char *FRAME[] =
-{
+static char *FRAME[] = {
 #ifdef MENU_THIN_FRAME
     "", "", "",
     "", "  ", "",
@@ -44,28 +42,28 @@ static char *FRAME[] =
     "", "  ", "",
     "", "", "",
 #endif				/* not MENU_THIN_FRAME */
-    "", ""};
+    "", ""
+};
 #define FRAME_WIDTH 2
 
 #define G_start
-/**/
-#define G_end    /**/
-
+ /**/
+#define G_end /**/
 #else				/* not KANJI_SYMBOLS */
-static char *N_FRAME[] =
-{
+static char *N_FRAME[] = {
     "+", "-", "+",
     "|", " ", "|",
     "+", "-", "+",
-    ":", ":"};
+    ":", ":"
+};
 #define FRAME_WIDTH 1
 
-static char *G_FRAME[] =
-{
+static char *G_FRAME[] = {
     "l", "q", "k",
     "x", " ", "x",
     "m", "q", "j",
-    ":", ":"};
+    ":", ":"
+};
 
 static char **FRAME = NULL;
 static int graph_mode = FALSE;
@@ -97,9 +95,10 @@ static int mSrchB(char c);
 static int mSrchN(char c);
 static int mSrchP(char c);
 #ifdef __EMX__
-static int	mPc(char c);
+static int mPc(char c);
 #endif
 
+/* *INDENT-OFF* */
 static int (*MenuKeymap[128]) (char c) = {
 /*  C-@     C-a     C-b     C-c     C-d     C-e     C-f     C-g      */
 #ifdef __EMX__
@@ -266,7 +265,7 @@ static int (*MenuPcKeymap[256])(char c)={
   mNull,  mNull,  mNull,  mNull,  mNull,  mNull,  mNull,  mNull	   // 248
 };
 #endif
-
+/* *INDENT-ON* */
 /* --- SelectMenu --- */
 
 static Menu SelectMenu;
@@ -281,11 +280,10 @@ static int smDelBuf(char c);
 
 static Menu MainMenu;
 #if LANG == JA
-static MenuItem MainMenuItem[] =
-{
-/* type        label         variabel value func     popup keys data  */
+static MenuItem MainMenuItem[] = {
+    /* type        label         variabel value func     popup keys data  */
     {MENU_FUNC, "         (b)", NULL, 0, backBf, NULL, "b", NULL},
-    {MENU_FUNC, "Хåե (s)", NULL, 0, selBuf, NULL, "s", NULL},
+    {MENU_POPUP, "Хåե (s)", NULL, 0, NULL, &SelectMenu, "s", NULL},
     {MENU_FUNC, "ɽ (v)", NULL, 0, vwSrc, NULL, "vV", NULL},
     {MENU_FUNC, "Խ (e)", NULL, 0, editBf, NULL, "eE", NULL},
     {MENU_FUNC, "¸ (S)", NULL, 0, svSrc, NULL, "S", NULL},
@@ -305,11 +303,10 @@ static MenuItem MainMenuItem[] =
     {MENU_END, "", NULL, 0, nulcmd, NULL, "", NULL},
 };
 #else				/* LANG != JA */
-static MenuItem MainMenuItem[] =
-{
-/* type        label           variable value func     popup keys data  */
+static MenuItem MainMenuItem[] = {
+    /* type        label           variable value func     popup keys data  */
     {MENU_FUNC, " Back         (b) ", NULL, 0, backBf, NULL, "b", NULL},
-    {MENU_FUNC, " Select Buffer(s) ", NULL, 0, selBuf, NULL, "s", NULL},
+    {MENU_POPUP, " Select Buffer(s) ", NULL, 0, NULL, &SelectMenu, "s", NULL},
     {MENU_FUNC, " View Source  (v) ", NULL, 0, vwSrc, NULL, "vV", NULL},
     {MENU_FUNC, " Edit Source  (e) ", NULL, 0, editBf, NULL, "eE", NULL},
     {MENU_FUNC, " Save Source  (S) ", NULL, 0, svSrc, NULL, "S", NULL},
@@ -332,8 +329,6 @@ static MenuItem MainMenuItem[] =
 
 /* --- MainMenu (END) --- */
 
-extern int w3mNFuncList;
-extern FuncList w3mFuncList[];
 static MenuList *w3mMenuList;
 
 static Menu *CurrentMenu = NULL;
@@ -343,7 +338,7 @@ static Menu *CurrentMenu = NULL;
 #define mvaddnstr(y, x, str, n) (move(y, x), addnstr_sup(str, n))
 
 void
-new_menu(Menu * menu, MenuItem * item)
+new_menu(Menu *menu, MenuItem *item)
 {
     int i, l;
     char *p;
@@ -362,7 +357,7 @@ new_menu(Menu * menu, MenuItem * item)
     if (item == NULL)
 	return;
 
-    for (i = 0; item[i].type != MENU_END; i++);
+    for (i = 0; item[i].type != MENU_END; i++) ;
     menu->nitem = i;
     menu->height = menu->nitem;
     for (i = 0; i < 128; i++)
@@ -372,8 +367,8 @@ new_menu(Menu * menu, MenuItem * item)
 	if ((p = item[i].keys) != NULL) {
 	    while (*p) {
 		if (IS_ASCII(*p)) {
-		    menu->keymap[(int) *p] = mSelect;
-		    menu->keyselect[(int) *p] = i;
+		    menu->keymap[(int)*p] = mSelect;
+		    menu->keyselect[(int)*p] = i;
 		}
 		p++;
 	    }
@@ -385,11 +380,11 @@ new_menu(Menu * menu, MenuItem * item)
 }
 
 void
-geom_menu(Menu * menu, int x, int y, int select)
+geom_menu(Menu *menu, int x, int y, int mselect)
 {
     int win_x, win_y, win_w, win_h;
 
-    menu->select = select;
+    menu->select = mselect;
 
     if (menu->width % FRAME_WIDTH)
 	menu->width = (menu->width / FRAME_WIDTH + 1) * FRAME_WIDTH;
@@ -407,7 +402,7 @@ geom_menu(Menu * menu, int x, int y, int select)
     }
     menu->x = win_x + FRAME_WIDTH;
 
-    win_y = menu->y - select - 1;
+    win_y = menu->y - mselect - 1;
     win_h = menu->height + 2;
     if (win_y + win_h > LASTLINE)
 	win_y = LASTLINE - win_h;
@@ -416,15 +411,15 @@ geom_menu(Menu * menu, int x, int y, int select)
 	if (win_y + win_h > LASTLINE) {
 	    win_h = LASTLINE - win_y;
 	    menu->height = win_h - 2;
-	    if (menu->height <= select)
-		menu->offset = select - menu->height + 1;
+	    if (menu->height <= mselect)
+		menu->offset = mselect - menu->height + 1;
 	}
     }
     menu->y = win_y + 1;
 }
 
 void
-draw_all_menu(Menu * menu)
+draw_all_menu(Menu *menu)
 {
     if (menu->parent != NULL)
 	draw_all_menu(menu->parent);
@@ -432,7 +427,7 @@ draw_all_menu(Menu * menu)
 }
 
 void
-draw_menu(Menu * menu)
+draw_menu(Menu *menu)
 {
     int x, y, w;
     int i, j;
@@ -508,32 +503,32 @@ draw_menu(Menu * menu)
 }
 
 void
-draw_menu_item(Menu * menu, int select)
+draw_menu_item(Menu *menu, int mselect)
 {
-    mvaddnstr(menu->y + select - menu->offset, menu->x,
-	      menu->item[select].label, menu->width);
+    mvaddnstr(menu->y + mselect - menu->offset, menu->x,
+	      menu->item[mselect].label, menu->width);
 }
 
 int
-select_menu(Menu * menu, int select)
+select_menu(Menu *menu, int mselect)
 {
-    if (select < 0 || select >= menu->nitem)
+    if (mselect < 0 || mselect >= menu->nitem)
 	return (MENU_NOTHING);
-    if (select < menu->offset)
-	up_menu(menu, menu->offset - select);
-    else if (select >= menu->offset + menu->height)
-	down_menu(menu, select - menu->offset - menu->height + 1);
+    if (mselect < menu->offset)
+	up_menu(menu, menu->offset - mselect);
+    else if (mselect >= menu->offset + menu->height)
+	down_menu(menu, mselect - menu->offset - menu->height + 1);
 
     if (menu->select >= menu->offset &&
 	menu->select < menu->offset + menu->height)
 	draw_menu_item(menu, menu->select);
-    menu->select = select;
+    menu->select = mselect;
     standout();
     draw_menu_item(menu, menu->select);
     standend();
-/* 
- * move(menu->cursorY, menu->cursorX); */
-    move(menu->y + select - menu->offset, menu->x);
+    /* 
+     * move(menu->cursorY, menu->cursorX); */
+    move(menu->y + mselect - menu->offset, menu->x);
     toggle_stand();
     refresh();
 
@@ -541,28 +536,26 @@ select_menu(Menu * menu, int select)
 }
 
 void
-goto_menu(Menu * menu, int select, int down)
+goto_menu(Menu *menu, int mselect, int down)
 {
     int select_in;
-    if (select >= menu->nitem)
-	select = menu->nitem - 1;
-    else if (select < 0)
-	select = 0;
-    select_in = select;
-    while (menu->item[select].type == MENU_NOP) {
+    if (mselect >= menu->nitem)
+	mselect = menu->nitem - 1;
+    else if (mselect < 0)
+	mselect = 0;
+    select_in = mselect;
+    while (menu->item[mselect].type == MENU_NOP) {
 	if (down > 0) {
-	    if (++select >= menu->nitem)
-	    {
+	    if (++mselect >= menu->nitem) {
 		down_menu(menu, select_in - menu->select);
-		select = menu->select;
+		mselect = menu->select;
 		break;
 	    }
 	}
 	else if (down < 0) {
-	    if (--select < 0)
-	    {
+	    if (--mselect < 0) {
 		up_menu(menu, menu->select - select_in);
-		select = menu->select;
+		mselect = menu->select;
 		break;
 	    }
 	}
@@ -570,11 +563,11 @@ goto_menu(Menu * menu, int select, int down)
 	    return;
 	}
     }
-    select_menu(menu, select);
+    select_menu(menu, mselect);
 }
 
 void
-up_menu(Menu * menu, int n)
+up_menu(Menu *menu, int n)
 {
     if (n < 0 || menu->offset == 0)
 	return;
@@ -586,7 +579,7 @@ up_menu(Menu * menu, int n)
 }
 
 void
-down_menu(Menu * menu, int n)
+down_menu(Menu *menu, int n)
 {
     if (n < 0 || menu->offset + menu->height == menu->nitem)
 	return;
@@ -598,10 +591,10 @@ down_menu(Menu * menu, int n)
 }
 
 int
-action_menu(Menu * menu)
+action_menu(Menu *menu)
 {
     char c;
-    int select;
+    int mselect;
     MenuItem item;
 
     if (menu->active == 0) {
@@ -613,31 +606,30 @@ action_menu(Menu * menu)
     select_menu(menu, menu->select);
 
     while (1) {
-#ifdef MOUSE
+#ifdef USE_MOUSE
 	if (use_mouse)
 	    mouse_active();
-#endif				/* MOUSE */
+#endif				/* USE_MOUSE */
 	c = getch();
-#ifdef MOUSE
+#ifdef USE_MOUSE
 	if (use_mouse)
 	    mouse_inactive();
 #if defined(USE_GPM) || defined(USE_SYSMOUSE)
 	if (c == X_MOUSE_SELECTED) {
-	    select = X_Mouse_Selection;
-	    if (select != MENU_NOTHING)
+	    mselect = X_Mouse_Selection;
+	    if (mselect != MENU_NOTHING)
 		break;
 	}
-#endif				/* defined(USE_GPM) || * * * * * *
-				 * defined(USE_SYSMOUSE) */
-#endif				/* MOUSE */
+#endif				/* defined(USE_GPM) || defined(USE_SYSMOUSE) */
+#endif				/* USE_MOUSE */
 	if (IS_ASCII(c)) {	/* Ascii */
-	    select = (*menu->keymap[(int) c]) (c);
-	    if (select != MENU_NOTHING)
+	    mselect = (*menu->keymap[(int)c]) (c);
+	    if (mselect != MENU_NOTHING)
 		break;
 	}
     }
-    if (select >= 0 && select < menu->nitem) {
-	item = menu->item[select];
+    if (mselect >= 0 && mselect < menu->nitem) {
+	item = menu->item[mselect];
 	if (item.type & MENU_POPUP) {
 	    popup_menu(menu, item.popup);
 	    return (1);
@@ -654,7 +646,7 @@ action_menu(Menu * menu)
 	    CurrentMenuData = NULL;
 	}
     }
-    else if (select == MENU_CLOSE) {
+    else if (mselect == MENU_CLOSE) {
 	if (menu->parent != NULL)
 	    menu->parent->active = 0;
     }
@@ -662,7 +654,7 @@ action_menu(Menu * menu)
 }
 
 void
-popup_menu(Menu * parent, Menu * menu)
+popup_menu(Menu *parent, Menu *menu)
 {
     int active = 1;
 
@@ -671,14 +663,14 @@ popup_menu(Menu * parent, Menu * menu)
     if (menu->active)
 	return;
 
-#ifdef MOUSE
+#ifdef USE_MOUSE
 #ifdef USE_GPM
     gpm_handler = gpm_process_menu_mouse;
 #endif				/* USE_GPM */
 #ifdef USE_SYSMOUSE
     sysm_handler = sysm_process_menu_mouse;
 #endif				/* USE_SYSMOUSE */
-#endif				/* MOUSE */
+#endif				/* USE_MOUSE */
     menu->parent = parent;
     menu->select = menu->initial;
     menu->offset = 0;
@@ -697,7 +689,7 @@ popup_menu(Menu * parent, Menu * menu)
     }
     menu->active = 0;
     CurrentMenu = parent;
-#ifdef MOUSE
+#ifdef USE_MOUSE
 #ifdef USE_GPM
     if (CurrentMenu == NULL)
 	gpm_handler = gpm_process_mouse;
@@ -706,11 +698,11 @@ popup_menu(Menu * parent, Menu * menu)
     if (CurrentMenu == NULL)
 	sysm_handler = sysm_process_mouse;
 #endif				/* USE_SYSMOUSE */
-#endif				/* MOUSE */
+#endif				/* USE_MOUSE */
 }
 
 void
-guess_menu_xy(Menu * parent, int width, int *x, int *y)
+guess_menu_xy(Menu *parent, int width, int *x, int *y)
 {
     *x = parent->x + parent->width + FRAME_WIDTH - 1;
     if (*x + width + FRAME_WIDTH > COLS) {
@@ -723,7 +715,7 @@ guess_menu_xy(Menu * parent, int width, int *x, int *y)
 }
 
 void
-new_option_menu(Menu * menu, char **label, int *variable, void (*func) ())
+new_option_menu(Menu *menu, char **label, int *variable, void (*func) ())
 {
     int i, nitem;
     char **p;
@@ -732,7 +724,7 @@ new_option_menu(Menu * menu, char **label, int *variable, void (*func) ())
     if (label == NULL || *label == NULL)
 	return;
 
-    for (i = 0, p = label; *p != NULL; i++, p++);
+    for (i = 0, p = label; *p != NULL; i++, p++) ;
     nitem = i;
 
     item = New_N(MenuItem, nitem + 1);
@@ -760,8 +752,8 @@ new_option_menu(Menu * menu, char **label, int *variable, void (*func) ())
 static int
 mPc(char c)
 {
-  c = getch();
-  return(MenuPcKeymap[(int)c](c));
+    c = getch();
+    return (MenuPcKeymap[(int)c] (c));
 }
 #endif
 
@@ -769,7 +761,7 @@ static int
 mEsc(char c)
 {
     c = getch();
-    return (MenuEscKeymap[(int) c] (c));
+    return (MenuEscKeymap[(int)c] (c));
 }
 
 static int
@@ -779,7 +771,7 @@ mEscB(char c)
     if (IS_DIGIT(c))
 	return (mEscD(c));
     else
-	return (MenuEscBKeymap[(int) c] (c));
+	return (MenuEscBKeymap[(int)c] (c));
 }
 
 static int
@@ -787,10 +779,10 @@ mEscD(char c)
 {
     int d;
 
-    d = (int) c - (int) '0';
+    d = (int)c - (int)'0';
     c = getch();
     if (IS_DIGIT(c)) {
-	d = d * 10 + (int) c - (int) '0';
+	d = d * 10 + (int)c - (int)'0';
 	c = getch();
     }
     if (c == '~')
@@ -809,7 +801,7 @@ static int
 mSelect(char c)
 {
     if (IS_ASCII(c))
-	return (select_menu(CurrentMenu, CurrentMenu->keyselect[(int) c]));
+	return (select_menu(CurrentMenu, CurrentMenu->keyselect[(int)c]));
     else
 	return (MENU_NOTHING);
 }
@@ -849,24 +841,24 @@ mTop(char c)
 static int
 mNext(char c)
 {
-    int select = CurrentMenu->select + CurrentMenu->height;
+    int mselect = CurrentMenu->select + CurrentMenu->height;
 
-    if (select >= CurrentMenu->nitem)
+    if (mselect >= CurrentMenu->nitem)
 	return mLast(c);
     down_menu(CurrentMenu, CurrentMenu->height);
-    goto_menu(CurrentMenu, select, -1);
+    goto_menu(CurrentMenu, mselect, -1);
     return (MENU_NOTHING);
 }
 
 static int
 mPrev(char c)
 {
-    int select = CurrentMenu->select - CurrentMenu->height;
+    int mselect = CurrentMenu->select - CurrentMenu->height;
 
-    if (select < 0)
+    if (mselect < 0)
 	return mTop(c);
     up_menu(CurrentMenu, CurrentMenu->height);
-    goto_menu(CurrentMenu, select, 1);
+    goto_menu(CurrentMenu, mselect, 1);
     return (MENU_NOTHING);
 }
 
@@ -875,7 +867,8 @@ mFore(char c)
 {
     if (CurrentMenu->select >= CurrentMenu->nitem - 1)
 	return (MENU_NOTHING);
-    goto_menu(CurrentMenu, (CurrentMenu->select + CurrentMenu->height - 1), (CurrentMenu->height + 1));
+    goto_menu(CurrentMenu, (CurrentMenu->select + CurrentMenu->height - 1),
+	      (CurrentMenu->height + 1));
     return (MENU_NOTHING);
 }
 
@@ -884,18 +877,19 @@ mBack(char c)
 {
     if (CurrentMenu->select <= 0)
 	return (MENU_NOTHING);
-    goto_menu(CurrentMenu, (CurrentMenu->select - CurrentMenu->height + 1), (-1 - CurrentMenu->height));
+    goto_menu(CurrentMenu, (CurrentMenu->select - CurrentMenu->height + 1),
+	      (-1 - CurrentMenu->height));
     return (MENU_NOTHING);
 }
 
 static int
 mOk(char c)
 {
-    int select = CurrentMenu->select;
+    int mselect = CurrentMenu->select;
 
-    if (CurrentMenu->item[select].type == MENU_NOP)
+    if (CurrentMenu->item[mselect].type == MENU_NOP)
 	return (MENU_NOTHING);
-    return (select);
+    return (mselect);
 }
 
 static int
@@ -924,22 +918,25 @@ static char *SearchString = NULL;
 int (*menuSearchRoutine) (Menu *, char *, int);
 
 static int
-menuForwardSearch (Menu* menu, char* str, int from)
+menuForwardSearch(Menu *menu, char *str, int from)
 {
     int i;
-    char* p;
-    if ((p = regexCompile (str, IgnoreCase)) != NULL) {
-	message (p, 0, 0);
+    char *p;
+    if ((p = regexCompile(str, IgnoreCase)) != NULL) {
+	message(p, 0, 0);
 	return -1;
     }
+    if (from < 0)
+	from = 0;
     for (i = from; i < menu->nitem; i++)
-	if (regexMatch (menu->item[i].label, 0, 0) == 1)
-            return i;
+	if (menu->item[i].type != MENU_NOP &&
+	    regexMatch(menu->item[i].label, 0, 1) == 1)
+	    return i;
     return -1;
 }
 
-int
-menu_search_forward (Menu* menu, int from)
+static int
+menu_search_forward(Menu *menu, int from)
 {
     char *str;
     int found;
@@ -947,43 +944,48 @@ menu_search_forward (Menu* menu, int from)
     if (str != NULL && *str == '\0')
 	str = SearchString;
     if (str == NULL || *str == '\0')
-	return (MENU_NOTHING);
+	return -1;
     SearchString = str;
-    found = menuForwardSearch (menu, SearchString, from);
-    if (WrapSearch && found == -1)
-        found = menuForwardSearch (menu, SearchString, 0);
     menuSearchRoutine = menuForwardSearch;
+    found = menuForwardSearch(menu, SearchString, from + 1);
+    if (WrapSearch && found == -1)
+	found = menuForwardSearch(menu, SearchString, 0);
     if (found >= 0)
-        return found;
-    return from;
+	return found;
+    disp_message("Not found", TRUE);
+    return -1;
 }
 
 static int
-mSrchF (char c)
+mSrchF(char c)
 {
-    int select;
-    select = menu_search_forward (CurrentMenu, CurrentMenu->select);
-    goto_menu (CurrentMenu, select, 1);
+    int mselect;
+    mselect = menu_search_forward(CurrentMenu, CurrentMenu->select);
+    if (mselect >= 0)
+	goto_menu(CurrentMenu, mselect, 1);
     return (MENU_NOTHING);
 }
 
 static int
-menuBackwardSearch (Menu* menu, char* str, int from)
+menuBackwardSearch(Menu *menu, char *str, int from)
 {
     int i;
-    char* p;
-    if ((p = regexCompile (str, IgnoreCase)) != NULL) {
-	message (p, 0, 0);
+    char *p;
+    if ((p = regexCompile(str, IgnoreCase)) != NULL) {
+	message(p, 0, 0);
 	return -1;
     }
-    for (i = from; i >= 0 ; i--)
-	if (regexMatch (menu->item[i].label, 0, 0) == 1)
-            return i;
+    if (from >= menu->nitem)
+	from = menu->nitem - 1;
+    for (i = from; i >= 0; i--)
+	if (menu->item[i].type != MENU_NOP &&
+	    regexMatch(menu->item[i].label, 0, 1) == 1)
+	    return i;
     return -1;
 }
 
-int
-menu_search_backward (Menu* menu, int from)
+static int
+menu_search_backward(Menu *menu, int from)
 {
     char *str;
     int found;
@@ -993,76 +995,73 @@ menu_search_backward (Menu* menu, int from)
     if (str == NULL || *str == '\0')
 	return (MENU_NOTHING);
     SearchString = str;
-    found = menuBackwardSearch (menu, SearchString, from);
-    if (WrapSearch && found == -1)
-        found = menuBackwardSearch (menu, SearchString, 0);
     menuSearchRoutine = menuBackwardSearch;
+    found = menuBackwardSearch(menu, SearchString, from - 1);
+    if (WrapSearch && found == -1)
+	found = menuBackwardSearch(menu, SearchString, menu->nitem);
     if (found >= 0)
-        return found;
-    return from;
+	return found;
+    disp_message("Not found", TRUE);
+    return -1;
 }
 
 static int
-mSrchB (char c)
+mSrchB(char c)
 {
-    int select;
-    select = menu_search_backward (CurrentMenu, CurrentMenu->select);
-    goto_menu (CurrentMenu, select, -1);
+    int mselect;
+    mselect = menu_search_backward(CurrentMenu, CurrentMenu->select);
+    if (mselect >= 0)
+	goto_menu(CurrentMenu, mselect, -1);
     return (MENU_NOTHING);
 }
 
 static int
-menu_search_next_previous (Menu* menu, int from, int reverse)
+menu_search_next_previous(Menu *menu, int from, int reverse)
 {
     int found;
-    int new_from;
-    static int (*routine[2]) (Menu *, char *, int) =
-    {
-	menuForwardSearch, menuBackwardSearch
-    };
+    static int (*routine[2]) (Menu *, char *, int) = {
+    menuForwardSearch, menuBackwardSearch};
 
     if (menuSearchRoutine == NULL) {
-	disp_message ("No previous regular expression", TRUE);
-	return from;
+	disp_message("No previous regular expression", TRUE);
+	return -1;
     }
-    addstr(menuSearchRoutine == menuForwardSearch ? "Forward: " : "Backward: ");
-    addstr(SearchString);
     if (reverse != 0)
 	reverse = 1;
     if (menuSearchRoutine == menuBackwardSearch)
 	reverse ^= 1;
-    new_from = from - reverse * 2 + 1;
-    if (new_from >=0 && new_from < menu->nitem)
-	found = (*routine[reverse]) (menu, SearchString, new_from);
-    else
-	found = (*routine[reverse]) (menu, SearchString, from);
-    if (WrapSearch && found == -1) {
-        found = (*routine[reverse]) (menu, SearchString, reverse * menu->nitem);
-    }
+    from += reverse ? -1 : 1;
+    found = (*routine[reverse]) (menu, SearchString, from);
+    if (WrapSearch && found == -1)
+	found =
+	    (*routine[reverse]) (menu, SearchString, reverse * menu->nitem);
     if (found >= 0)
-        return found;
-    return from;
+	return found;
+    disp_message("Not found", TRUE);
+    return -1;
 }
 
 static int
-mSrchN (char c)
+mSrchN(char c)
 {
-    int select;
-    select = menu_search_next_previous (CurrentMenu, CurrentMenu->select, 0);
-    goto_menu (CurrentMenu, select, 1);
+    int mselect;
+    mselect = menu_search_next_previous(CurrentMenu, CurrentMenu->select, 0);
+    if (mselect >= 0)
+	goto_menu(CurrentMenu, mselect, 1);
     return (MENU_NOTHING);
 }
 
 static int
-mSrchP (char c)
+mSrchP(char c)
 {
-    int select;
-    select = menu_search_next_previous (CurrentMenu, CurrentMenu->select, 1);
-    goto_menu (CurrentMenu, select, -1);
+    int mselect;
+    mselect = menu_search_next_previous(CurrentMenu, CurrentMenu->select, 1);
+    if (mselect >= 0)
+	goto_menu(CurrentMenu, mselect, -1);
     return (MENU_NOTHING);
 }
 
-#ifdef MOUSE
+#ifdef USE_MOUSE
 #define MOUSE_BTN1_DOWN 0
 #define MOUSE_BTN2_DOWN 1
 #define MOUSE_BTN3_DOWN 2
@@ -1078,7 +1077,7 @@ static int
 process_mMouse(int btn, int x, int y)
 {
     Menu *menu;
-    int select;
+    int mselect;
     static int press_btn = MOUSE_BTN_RESET, press_x, press_y;
     char c = ' ';
 
@@ -1088,12 +1087,10 @@ process_mMouse(int btn, int x, int y)
 	return (MENU_NOTHING);
 
     if (btn == MOUSE_BTN_UP) {
-	if (press_btn == MOUSE_BTN1_DOWN ||
-	    press_btn == MOUSE_BTN3_DOWN) {
+	if (press_btn == MOUSE_BTN1_DOWN || press_btn == MOUSE_BTN3_DOWN) {
 	    if (x < menu->x - FRAME_WIDTH ||
 		x >= menu->x + menu->width + FRAME_WIDTH ||
-		y < menu->y - 1 ||
-		y >= menu->y + menu->height + 1) {
+		y < menu->y - 1 || y >= menu->y + menu->height + 1) {
 		return (MENU_CANCEL);
 	    }
 	    else if ((x >= menu->x - FRAME_WIDTH &&
@@ -1111,10 +1108,10 @@ process_mMouse(int btn, int x, int y)
 		return (MENU_NOTHING);
 	    }
 	    else {
-		select = y - menu->y + menu->offset;
-		if (menu->item[select].type == MENU_NOP)
+		mselect = y - menu->y + menu->offset;
+		if (menu->item[mselect].type == MENU_NOP)
 		    return (MENU_NOTHING);
-		return (select_menu(menu, select));
+		return (select_menu(menu, mselect));
 	    }
 	}
     }
@@ -1131,11 +1128,11 @@ mMouse(char c)
 {
     int btn, x, y;
 
-    btn = (unsigned char) getch() - 32;
-    x = (unsigned char) getch() - 33;
+    btn = (unsigned char)getch() - 32;
+    x = (unsigned char)getch() - 33;
     if (x < 0)
 	x += 0x100;
-    y = (unsigned char) getch() - 33;
+    y = (unsigned char)getch() - 33;
     if (y < 0)
 	y += 0x100;
 
@@ -1148,7 +1145,7 @@ mMouse(char c)
 static int
 gpm_process_menu_mouse(Gpm_Event * event, void *data)
 {
-    int btn, x, y;
+    int btn = MOUSE_BTN_RESET, x, y;
     if (event->type & GPM_UP)
 	btn = MOUSE_BTN_UP;
     else if (event->type & GPM_DOWN) {
@@ -1196,13 +1193,13 @@ sysm_process_menu_mouse(int x, int y, int nbs, int obs)
     return X_MOUSE_SELECTED;
 }
 #endif				/* USE_SYSMOUSE */
-#else				/* not MOUSE */
+#else				/* not USE_MOUSE */
 static int
 mMouse(char c)
 {
     return (MENU_NOTHING);
 }
-#endif				/* not MOUSE */
+#endif				/* not USE_MOUSE */
 
 /* --- MenuFunctions (END) --- */
 
@@ -1213,7 +1210,7 @@ popupMenu(int x, int y, Menu *menu)
 {
     initSelectMenu();
 
-    menu->cursorX = Currentbuf->cursorX;
+    menu->cursorX = Currentbuf->cursorX + Currentbuf->rootX;
     menu->cursorY = Currentbuf->cursorY;
     menu->x = x + FRAME_WIDTH + 1;
     menu->y = y + 2;
@@ -1241,13 +1238,20 @@ mainMn(void)
 	    return;
 	menu = w3mMenuList[n].menu;
     }
-    popupMenu(Currentbuf->cursorX, Currentbuf->cursorY, menu);
+    popupMenu(Currentbuf->cursorX + Currentbuf->rootX, Currentbuf->cursorY,
+	      menu);
 }
 
 /* --- MainMenu (END) --- */
 
 /* --- SelectMenu --- */
 
+void
+selMn(void)
+{
+    popupMenu(Currentbuf->cursorX, Currentbuf->cursorY, &SelectMenu);
+}
+
 static void
 initSelectMenu(void)
 {
@@ -1273,7 +1277,8 @@ initSelectMenu(void)
 	    case SCM_LOCAL_CGI:
 		if (strcmp(buf->currentURL.file, "-")) {
 		    Strcat_char(str, ' ');
-		    Strcat_charp(str, buf->filename);
+		    Strcat_charp(str,
+				 conv_from_system(buf->currentURL.real_file));
 		}
 		break;
 	    case SCM_UNKNOWN:
@@ -1306,7 +1311,7 @@ initSelectMenu(void)
 
     new_option_menu(&SelectMenu, label, &SelectV, smChBuf);
     SelectMenu.initial = SelectV;
-    SelectMenu.cursorX = Currentbuf->cursorX;
+    SelectMenu.cursorX = Currentbuf->cursorX + Currentbuf->rootX;
     SelectMenu.cursorY = Currentbuf->cursorY;
     SelectMenu.keymap['D'] = smDelBuf;
     SelectMenu.item[nitem].type = MENU_NOP;
@@ -1320,7 +1325,7 @@ smChBuf(void)
 
     if (SelectV < 0 || SelectV >= SelectMenu.nitem)
 	return;
-    for (i = 0, buf = Firstbuf; i < SelectV; i++, buf = buf->nextBuffer);
+    for (i = 0, buf = Firstbuf; i < SelectV; i++, buf = buf->nextBuffer) ;
     Currentbuf = buf;
     if (clear_buffer) {
 	for (buf = Firstbuf; buf != NULL; buf = buf->nextBuffer)
@@ -1329,13 +1334,15 @@ smChBuf(void)
 }
 
 static int
-smDelBuf(char c) {
-    int i, x, y, select;
+smDelBuf(char c)
+{
+    int i, x, y, mselect;
     Buffer *buf;
 
     if (CurrentMenu->select < 0 || CurrentMenu->select >= SelectMenu.nitem)
 	return (MENU_NOTHING);
-    for (i = 0, buf = Firstbuf; i < CurrentMenu->select; i++, buf = buf->nextBuffer);
+    for (i = 0, buf = Firstbuf; i < CurrentMenu->select;
+	 i++, buf = buf->nextBuffer) ;
     if (Currentbuf == buf)
 	Currentbuf = buf->nextBuffer;
     Firstbuf = deleteBuffer(Firstbuf, buf);
@@ -1348,7 +1355,7 @@ smDelBuf(char c) {
 
     x = CurrentMenu->x;
     y = CurrentMenu->y;
-    select = CurrentMenu->select;
+    mselect = CurrentMenu->select;
 
     initSelectMenu();
 
@@ -1357,7 +1364,7 @@ smDelBuf(char c) {
 
     geom_menu(CurrentMenu, x, y, 0);
 
-    CurrentMenu->select = (select <= CurrentMenu->nitem - 2) ? select
+    CurrentMenu->select = (mselect <= CurrentMenu->nitem - 2) ? mselect
 	: (CurrentMenu->nitem - 2);
 
     displayBuffer(Currentbuf, B_FORCE_REDRAW);
@@ -1371,7 +1378,8 @@ smDelBuf(char c) {
 /* --- OptionMenu --- */
 
 void
-optionMenu(int x, int y, char **label, int *variable, int initial, void (*func) ())
+optionMenu(int x, int y, char **label, int *variable, int initial,
+	   void (*func) ())
 {
     Menu menu;
 
@@ -1395,8 +1403,8 @@ initMenu(void)
     FILE *mf;
     Str line;
     char *p, *s;
-    int in_menu, nmenu, nitem, type;
-    MenuItem *item;
+    int in_menu, nmenu = 0, nitem = 0, type;
+    MenuItem *item = NULL;
     MenuList *list;
 
     w3mMenuList = New_N(MenuList, 3);
@@ -1411,9 +1419,6 @@ initMenu(void)
     if ((mf = fopen(rcFile(MENU_FILE), "rt")) == NULL)
 	goto create_menu;
 
-    if (!w3mNFuncList)
-	w3mNFuncList = countFuncList(w3mFuncList);
-
     in_menu = 0;
     while (!feof(mf)) {
 	line = Strfgets(mf);
@@ -1465,7 +1470,7 @@ initMenu(void)
 }
 
 int
-setMenuItem(MenuItem * item, char *type, char *line)
+setMenuItem(MenuItem *item, char *type, char *line)
 {
     char *label, *func, *popup, *keys, *data;
     int f;
@@ -1491,7 +1496,7 @@ setMenuItem(MenuItem * item, char *type, char *line)
 	    return -1;
 	item->type = MENU_FUNC;
 	item->label = label;
-	f = getFuncList(func, w3mFuncList, w3mNFuncList);
+	f = getFuncList(func);
 	item->func = w3mFuncList[(f >= 0) ? f : FUNCNAME_nulcmd].func;
 	item->keys = keys;
 	item->data = data;
@@ -1515,12 +1520,12 @@ setMenuItem(MenuItem * item, char *type, char *line)
 }
 
 int
-addMenuList(MenuList ** mlist, char *id)
+addMenuList(MenuList **mlist, char *id)
 {
     int n;
     MenuList *list = *mlist;
 
-    for (n = 0; list->id != NULL; list++, n++);
+    for (n = 0; list->id != NULL; list++, n++) ;
     *mlist = New_Reuse(MenuList, *mlist, (n + 2));
     list = *mlist + n;
     list->id = id;
@@ -1531,7 +1536,7 @@ addMenuList(MenuList ** mlist, char *id)
 }
 
 int
-getMenuN(MenuList * list, char *id)
+getMenuN(MenuList *list, char *id)
 {
     int n;
 
@@ -1544,4 +1549,4 @@ getMenuN(MenuList * list, char *id)
 
 /* --- InitMenu (END) --- */
 
-#endif				/* MENU */
+#endif				/* USE_MENU */
