@@ -1,4 +1,4 @@
-/* $Id: terms.c,v 1.3 2001/11/15 00:32:13 a-ito Exp $ */
+/* $Id: terms.c,v 1.25 2001/12/17 15:39:32 ukai Exp $ */
 /* 
  * An original curses library for EUC-kanji by Akinori ITO,     December 1989
  * revised by Akinori ITO, January 1995
@@ -12,7 +12,11 @@
 #include <unistd.h>
 #include "config.h"
 #include <string.h>
-#ifdef MOUSE
+#ifdef HAVE_SYS_SELECT_H
+#include <sys/select.h>
+#endif
+#include <sys/ioctl.h>
+#ifdef USE_MOUSE
 #ifdef USE_GPM
 #include <gpm.h>
 #endif				/* USE_GPM */
@@ -29,14 +33,16 @@ static int cwidth = 8, cheight = 16;
 static int xpix, ypix, nbs, obs = 0;
 #endif				/* use_SYSMOUSE */
 
-static int is_xterm = 0;
+#ifndef __CYGWIN__
+static
+#endif
+int is_xterm = 0;
+
 void mouse_init(), mouse_end();
 int mouseActive = 0;
-#endif				/* MOUSE */
+#endif				/* USE_MOUSE */
 
-#ifdef AIX
-#include <sys/select.h>
-#endif				/* AIX */
+static int tty;
 
 #include "terms.h"
 #include "fm.h"
@@ -45,13 +51,260 @@ int mouseActive = 0;
 #ifdef __EMX__
 #define INCL_DOSNLS
 #include <os2.h>
-#include <sys/select.h>
 
 #ifndef JP_CHARSET
-extern int	CodePage;
+extern int CodePage;
 #endif				/* !JP_CHARSET */
 #endif				/* __EMX__ */
 
+#if defined(__CYGWIN__)
+#include <windows.h>
+static HANDLE hConIn;
+static int isWin95;
+static int isWinConsole;
+static char *ConInV;
+static int iConIn, nConIn, nConInMax;
+#ifdef USE_MOUSE
+static MOUSE_EVENT_RECORD lastConMouse;
+#endif
+
+void
+check_win32_console(void)
+{
+    char *tty;
+
+    tty = ttyname(1);
+    if (!strncmp(tty, "/dev/con", 8)) {
+	isWinConsole = TRUE;
+    }
+    else {
+	isWinConsole = FALSE;
+    }
+}
+
+void
+init_win32_console_handle(void)
+{
+    OSVERSIONINFO winVersionInfo;
+
+    check_win32_console();
+    winVersionInfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
+    if (GetVersionEx(&winVersionInfo) == 0) {
+	fprintf(stderr, "can't get Windows version information.\n");
+	exit(1);
+    }
+    if (winVersionInfo.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS) {
+	isWin95 = 1;
+    }
+    hConIn = INVALID_HANDLE_VALUE;
+    if (isWin95) {
+	if (isWinConsole) {
+	    if (isatty(0)) {
+		hConIn = GetStdHandle(STD_INPUT_HANDLE);
+	    }
+	    else {
+		hConIn = CreateFile("CONIN$", GENERIC_READ,
+				    FILE_SHARE_READ,
+				    NULL, OPEN_EXISTING, 0, NULL);
+	    }
+	}
+    }
+}
+
+static void
+expand_win32_console_input_buffer(int n)
+{
+    if (nConIn + n >= nConInMax) {
+	char *oldv;
+
+	nConInMax = ((nConIn + n) / 2 + 1) * 3;
+	oldv = ConInV;
+	ConInV = GC_MALLOC_ATOMIC(nConInMax);
+	memcpy(ConInV, oldv, nConIn);
+    }
+}
+
+static int
+read_win32_console_input(void)
+{
+    INPUT_RECORD rec;
+    DWORD nevents;
+
+    if (PeekConsoleInput(hConIn, &rec, 1, &nevents) && nevents) {
+	switch (rec.EventType) {
+	case KEY_EVENT:
+	    expand_win32_console_input_buffer(3);
+
+	    if (ReadConsole(hConIn, &ConInV[nConIn], 1, &nevents, NULL)) {
+		nConIn += nevents;
+		return nevents;
+	    }
+
+	    break;
+#ifdef USE_MOUSE
+	case MOUSE_EVENT:
+	    if ((lastConMouse.dwButtonState ^ rec.Event.MouseEvent.
+		 dwButtonState) & ~(~0 << 5)) {
+		int down;
+		MOUSE_EVENT_RECORD *mer;
+		INPUT_RECORD dummy;
+
+		expand_win32_console_input_buffer(6);
+		mer = &rec.Event.MouseEvent;
+		ConInV[nConIn] = '\033';
+		ConInV[nConIn + 1] = '[';
+		ConInV[nConIn + 2] = 'M';
+
+		if (~(mer->dwButtonState) & lastConMouse.
+		    dwButtonState & ~(~0 << 5))
+		    ConInV[nConIn + 3] = MOUSE_BTN_UP + ' ';
+		else if (!
+			 (down =
+			  mer->dwButtonState & ~lastConMouse.
+			  dwButtonState & ~(~0 << 5)))
+		    break;
+		else
+		    ConInV[nConIn + 3] = (down & (1 << 0) ? MOUSE_BTN1_DOWN :
+					  down & (1 << 1) ? MOUSE_BTN3_DOWN :
+					  down & (1 << 2) ? MOUSE_BTN2_DOWN :
+					  down & (1 << 3) ?
+					  MOUSE_BTN4_DOWN_XTERM :
+					  MOUSE_BTN5_DOWN_XTERM) + ' ';
+
+		ConInV[nConIn + 4] = mer->dwMousePosition.X + '!';
+		ConInV[nConIn + 5] = mer->dwMousePosition.Y + '!';
+		nConIn += 6;
+		lastConMouse = *mer;
+		ReadConsoleInput(hConIn, &rec, 1, &nevents);
+		return 6;
+	    }
+#endif
+	default:
+	    break;
+	}
+
+	ReadConsoleInput(hConIn, &rec, 1, &nevents);
+    }
+    return 0;
+}
+
+int
+read_win32_console(char *s, int n)
+{
+    KEY_EVENT_RECORD *ker;
+
+    if (hConIn == INVALID_HANDLE_VALUE)
+	return read(tty, s, n);
+
+    if (n > 0)
+	for (;;) {
+	    if (iConIn < nConIn) {
+		if (n > nConIn - iConIn)
+		    n = nConIn - iConIn;
+
+		memcpy(s, ConInV, n);
+
+		if ((iConIn += n) >= nConIn)
+		    iConIn = nConIn = 0;
+
+		break;
+	    }
+
+	    iConIn = nConIn = 0;
+
+	    while (!read_win32_console_input()) ;
+	}
+
+    return n;
+}
+
+static int
+cmp_tv(const struct timeval *tv_a, const struct timeval *tv_b)
+{
+    return ((tv_a->tv_sec < tv_b->tv_sec) ? -1 :
+	    (tv_a->tv_sec > tv_b->tv_sec) ? 1 :
+	    (tv_a->tv_usec < tv_b->tv_usec) ? -1 :
+	    (tv_a->tv_usec > tv_b->tv_usec) ? 1 : 0);
+}
+
+static int
+subtract_tv(struct timeval *dst, const struct timeval *src)
+{
+    if ((dst->tv_usec -= src->tv_usec) < 0) {
+	--(dst->tv_sec);
+	dst->tv_usec += 1000000;
+    }
+
+    return ((dst->tv_sec -= src->tv_sec) < 0 ? -1 :
+	    !dst->tv_sec ? (dst->tv_usec <
+			    0 ? -1 : !dst->tv_usec ? 0 : 1) : 1);
+}
+
+int
+select_or_poll_win32_console(int n, fd_set * rfds, fd_set * wfds,
+			     fd_set * efds, struct timeval *tout)
+{
+    int m;
+    DWORD nevents;
+
+    if ((m = select(n, rfds, wfds, efds, tout)) < 0)
+	return m;
+
+    if (iConIn < nConIn) {
+	FD_SET(tty, rfds);
+	++m;
+    }
+    else {
+	iConIn = nConIn = 0;
+
+	while (GetNumberOfConsoleInputEvents(hConIn, &nevents) && nevents)
+	    read_win32_console_input();
+
+	if (nConIn) {
+	    FD_SET(tty, rfds);
+	    ++m;
+	}
+    }
+
+    return m;
+}
+
+int
+select_win32_console(int n, fd_set * rfds, fd_set * wfds, fd_set * efds,
+		     struct timeval *tout)
+{
+    static struct timeval polltv = { 0, 1000000 / CLOCKS_PER_SEC };
+    int m;
+    struct timeval tv;
+
+    if (hConIn == INVALID_HANDLE_VALUE || tty < 0 || tty >= n || !rfds
+	|| !FD_ISSET(tty, rfds))
+	return select(n, rfds, wfds, efds, tout);
+
+    FD_CLR(tty, rfds);
+
+    if (tout) {
+	while (cmp_tv(tout, &polltv) > 0) {
+	    tv = polltv;
+
+	    if ((m = select_or_poll_win32_console(n, rfds, wfds, efds, &tv)))
+		return m;
+
+	    subtract_tv(tout, &polltv);
+	}
+
+	return select_or_poll_win32_console(n, rfds, wfds, efds, tout);
+    }
+    else
+	for (;;) {
+	    tv = polltv;
+
+	    if ((m = select_or_poll_win32_console(n, rfds, wfds, efds, &tv)))
+		return m;
+	}
+}
+#endif
+
 char *getenv(const char *);
 MySignalHandler reset_exit(SIGNAL_ARG), error_dump(SIGNAL_ARG);
 void setlinescols(void);
@@ -61,17 +314,16 @@ void flush_tty();
 #define SIGIOT SIGABRT
 #endif				/* not SIGIOT */
 
-#ifdef TERMIO
-#include        <sys/ioctl.h>
-#include        <termio.h>
+#ifdef HAVE_TERMIO_H
+#include <termio.h>
 typedef struct termio TerminalMode;
 #define TerminalSet(fd,x)       ioctl(fd,TCSETA,x)
 #define TerminalGet(fd,x)       ioctl(fd,TCGETA,x)
 #define MODEFLAG(d)     ((d).c_lflag)
 #define IMODEFLAG(d)    ((d).c_iflag)
-#endif				/* TERMIO */
+#endif				/* HAVE_TERMIO_H */
 
-#ifdef TERMIOS
+#ifdef HAVE_TERMIOS_H
 #include <termios.h>
 #include <unistd.h>
 typedef struct termios TerminalMode;
@@ -79,16 +331,15 @@ typedef struct termios TerminalMode;
 #define TerminalGet(fd,x)       tcgetattr(fd,x)
 #define MODEFLAG(d)     ((d).c_lflag)
 #define IMODEFLAG(d)    ((d).c_iflag)
-#endif				/* TERMIOS */
+#endif				/* HAVE_TERMIOS_H */
 
-#ifdef SGTTY
-#include        <sys/ioctl.h>
-#include        <sgtty.h>
+#ifdef HAVE_SGTTY_H
+#include <sgtty.h>
 typedef struct sgttyb TerminalMode;
 #define TerminalSet(fd,x)       ioctl(fd,TIOCSETP,x)
 #define TerminalGet(fd,x)       ioctl(fd,TIOCGETP,x)
 #define MODEFLAG(d)     ((d).sg_flags)
-#endif				/* SGTTY */
+#endif				/* HAVE_SGTTY_H */
 
 #define MAX_LINE        200
 #define MAX_COLUMN      400
@@ -127,7 +378,7 @@ typedef struct sgttyb TerminalMode;
 
 #define S_COLORED       0xf00
 
-#ifdef BG_COLOR
+#ifdef USE_BG_COLOR
 /* Background Color */
 #define COL_BCOLOR      0xf000
 #define COL_BBLACK      0x8000
@@ -141,7 +392,7 @@ typedef struct sgttyb TerminalMode;
 #define COL_BTERM       0x0000
 
 #define S_BCOLORED      0xf000
-#endif				/* BG_COLOR */
+#endif				/* USE_BG_COLOR */
 
 
 #define S_GRAPHICS      0x10
@@ -180,10 +431,13 @@ static
 char bp[1024], funcstr[256];
 
 char *T_cd, *T_ce, *T_kr, *T_kl, *T_cr, *T_bt, *T_ta, *T_sc, *T_rc,
-*T_so, *T_se, *T_us, *T_ue, *T_cl, *T_cm, *T_al, *T_sr, *T_md, *T_me,
-*T_ti, *T_te, *T_nd, *T_as, *T_ae, *T_eA, *T_ac, *T_op;
+    *T_so, *T_se, *T_us, *T_ue, *T_cl, *T_cm, *T_al, *T_sr, *T_md, *T_me,
+    *T_ti, *T_te, *T_nd, *T_as, *T_ae, *T_eA, *T_ac, *T_op;
 
 int LINES, COLS;
+#if defined(CYGWIN) && LANG == JA
+int LASTLINE;
+#endif				/* defined(CYGWIN) && LANG == JA */
 static int max_LINES = 0, max_COLS = 0;
 static int tab_step = 8;
 static int CurLine, CurColumn;
@@ -205,7 +459,7 @@ void clear(), wrap(), touch_line(), touch_column(int);
 void switch_wchar(FILE *);
 void switch_ascii(FILE *);
 #endif
-void need_clrtoeol(void), clrtoeol(void);
+void need_clrtoeol(void), clrtoeol(void);	/* conflicts with curs_clear(3)? */
 
 int write1(char);
 
@@ -219,33 +473,53 @@ writestr(char *s)
 
 #define MOVE(line,column)       writestr(tgoto(T_cm,column,line));
 
+#ifdef USE_MOUSE
+static struct mouse_term_info {
+    char *term;
+    int flag;
+} xterm_mouse_term[] = {
+    {
+    "xterm", NEED_XTERM_ON | NEED_XTERM_OFF}, {
+    "kterm", NEED_XTERM_ON | NEED_XTERM_OFF}, {
+    "rxvt", NEED_XTERM_ON | NEED_XTERM_OFF},
+#ifdef __CYGWIN__
+    {
+    "cygwin", NEED_XTERM_ON},
+#endif
+    {
+    NULL, 0}
+};
+#endif
+
 int
 set_tty(void)
 {
     char *ttyn;
-#ifdef MOUSE
-    char *term;
-#endif
 
     if (isatty(0))		/* stdin */
 	ttyn = ttyname(0);
     else
-#ifndef __EMX__
-	ttyn = "/dev/tty";
-#else				/* __EMX__ */
-	ttyn = "con";
-#endif				/* __EMX__ */
+	ttyn = DEV_TTY_PATH;
     tty = open(ttyn, O_RDWR);
     if (tty < 0) {
 	/* use stderr instead of stdin... is it OK???? */
 	tty = 2;
     }
     ttyf = fdopen(tty, "w");
+#ifdef __CYGWIN__
+    init_win32_console_handle();
+#endif
     TerminalGet(tty, &d_ioval);
-#ifdef MOUSE
-    term = getenv("TERM");
-    if (!strncmp(term, "kterm", 5) || !strncmp(term, "xterm", 5)) {
-	is_xterm = 1;
+#ifdef USE_MOUSE
+    {
+	char *term = getenv("TERM");
+	struct mouse_term_info *p;
+	for (p = xterm_mouse_term; p->term != NULL; p++) {
+	    if (!strncmp(term, p->term, strlen(p->term))) {
+		is_xterm = p->flag;
+		break;
+	    }
+	}
     }
 #endif
     return 0;
@@ -258,12 +532,13 @@ ttymode_set(int mode, int imode)
 
     TerminalGet(tty, &ioval);
     MODEFLAG(ioval) |= mode;
-#ifndef SGTTY
+#ifndef HAVE_SGTTY_H
     IMODEFLAG(ioval) |= imode;
-#endif				/* not SGTTY */
+#endif				/* not HAVE_SGTTY_H */
 
     while (TerminalSet(tty, &ioval) == -1) {
-	if (errno == EINTR || errno == EAGAIN) continue;
+	if (errno == EINTR || errno == EAGAIN)
+	    continue;
 	printf("Error occured while set %x: errno=%d\n", mode, errno);
 	reset_exit(SIGNAL_ARGLIST);
     }
@@ -276,18 +551,19 @@ ttymode_reset(int mode, int imode)
 
     TerminalGet(tty, &ioval);
     MODEFLAG(ioval) &= ~mode;
-#ifndef SGTTY
+#ifndef HAVE_SGTTY_H
     IMODEFLAG(ioval) &= ~imode;
-#endif				/* not SGTTY */
+#endif				/* not HAVE_SGTTY_H */
 
     while (TerminalSet(tty, &ioval) == -1) {
-	if (errno == EINTR || errno == EAGAIN) continue;
+	if (errno == EINTR || errno == EAGAIN)
+	    continue;
 	printf("Error occured while reset %x: errno=%d\n", mode, errno);
 	reset_exit(SIGNAL_ARGLIST);
     }
 }
 
-#ifndef SGTTY
+#ifndef HAVE_SGTTY_H
 void
 set_cc(int spec, int val)
 {
@@ -296,12 +572,13 @@ set_cc(int spec, int val)
     TerminalGet(tty, &ioval);
     ioval.c_cc[spec] = val;
     while (TerminalSet(tty, &ioval) == -1) {
-	if (errno == EINTR || errno == EAGAIN) continue;
+	if (errno == EINTR || errno == EAGAIN)
+	    continue;
 	printf("Error occured: errno=%d\n", errno);
 	reset_exit(SIGNAL_ARGLIST);
     }
 }
-#endif				/* not SGTTY */
+#endif				/* not HAVE_SGTTY_H */
 
 void
 close_tty(void)
@@ -314,9 +591,9 @@ reset_tty(void)
 {
     if (DisplayCode != CODE_EUC && DisplayCode != CODE_SJIS)
 #if defined(__EMX__)&&!defined(JP_CHARSET)
-      if(!CodePage)
+	if (!CodePage)
 #endif
-	writestr("\033(B");	/* designate US_ASCII */
+	    writestr("\033(B");	/* designate US_ASCII */
     writestr(T_op);		/* turn off */
     writestr(T_me);
     if (!Do_not_use_ti_te) {
@@ -335,10 +612,10 @@ MySignalHandler
 reset_exit(SIGNAL_ARG)
 {
     reset_tty();
-#ifdef MOUSE
+#ifdef USE_MOUSE
     if (mouseActive)
 	mouse_end();
-#endif				/* MOUSE */
+#endif				/* USE_MOUSE */
     w3m_exit(0);
     SIGNAL_RETURN;
 }
@@ -365,7 +642,7 @@ set_int(void)
 #ifdef	SIGBUS
     signal(SIGBUS, error_dump);
 #endif				/* SIGBUS */
-/* signal(SIGSEGV, error_dump); */
+    /* signal(SIGSEGV, error_dump); */
 }
 
 
@@ -375,14 +652,14 @@ setgraphchar(void)
     int c, i, n;
 
     for (c = 0; c < 96; c++)
-	gcmap[c] = (char) (c + ' ');
+	gcmap[c] = (char)(c + ' ');
 
     if (!T_ac)
 	return;
 
     n = strlen(T_ac);
     for (i = 0; i < n - 1; i += 2) {
-	c = (unsigned) T_ac[i] - ' ';
+	c = (unsigned)T_ac[i] - ' ';
 	if (c >= 0 && c < 96)
 	    gcmap[c] = T_ac[i + 1];
     }
@@ -390,7 +667,7 @@ setgraphchar(void)
 
 #define graphchar(c) (((unsigned)(c)>=' ' && (unsigned)(c)<128)? gcmap[(c)-' '] : (c))
 
-#define GETSTR(v,s) {v = pt; suc = tgetstr(s,&pt); if (!suc) v = ""; else v = allocStr(suc,0); }
+#define GETSTR(v,s) {v = pt; suc = tgetstr(s,&pt); if (!suc) v = ""; else v = allocStr(suc, -1); }
 
 void
 getTCstr(void)
@@ -400,22 +677,18 @@ getTCstr(void)
     char *pt = funcstr;
     int r;
 
-#ifdef	__DJGPP__
-    ent = getenv("TERM") ? getenv("TERM") : "dosansi";
-#else
-    ent = getenv("TERM");
-#endif				/* __DJGPP__ */
+    ent = getenv("TERM") ? getenv("TERM") : DEFAULT_TERM;
+    if (ent == NULL) {
+	fprintf(stderr, "TERM is not set\n");
+	reset_exit(SIGNAL_ARGLIST);
+    }
+
     r = tgetent(bp, ent);
     if (r != 1) {
 	/* Can't find termcap entry */
 	fprintf(stderr, "Can't find termcap entry %s\n", ent);
 	reset_exit(SIGNAL_ARGLIST);
     }
-#ifdef MOUSE
-   if (tgetstr("Km", &pt)) {
-	is_xterm = 1;	
-   } 
-#endif
 
     GETSTR(T_ce, "ce");		/* clear to the end of line */
     GETSTR(T_cd, "cd");		/* clear to the end of display */
@@ -452,15 +725,15 @@ getTCstr(void)
     GETSTR(T_as, "as");		/* alternative (graphic) charset start */
     GETSTR(T_ae, "ae");		/* alternative (graphic) charset end */
     GETSTR(T_ac, "ac");		/* graphics charset pairs */
-    GETSTR(T_op, "op");		/* set default color pair to its original
-				 * * * * * * * value */
+    GETSTR(T_op, "op");		/* set default color pair to its original value */
 #if defined( CYGWIN ) && CYGWIN < 1
-/* for TERM=pcansi on MS-DOS prompt.
+    /* for TERM=pcansi on MS-DOS prompt. */
+#if 0
     T_eA = "";
     T_as = "\033[12m";
     T_ae = "\033[10m";
     T_ac = "l\001k\002m\003j\004x\005q\006n\020a\024v\025w\026u\027t\031";
-*/
+#endif
     T_eA = "";
     T_as = "";
     T_ae = "";
@@ -472,10 +745,6 @@ getTCstr(void)
     setgraphchar();
 }
 
-#ifndef TIOCGWINSZ
-#include <sys/ioctl.h>
-#endif				/* not TIOCGWINSZ */
-
 void
 setlinescols(void)
 {
@@ -496,15 +765,15 @@ setlinescols(void)
 	    }
 	}
 #ifndef JP_CHARSET
-	else{
-	    ULONG CpList[8],CpSize;
-	    CodePage=-1;
-	    if(!DosQueryCp(sizeof(CpList),CpList,&CpSize))
-		CodePage=*CpList;
+	else {
+	    ULONG CpList[8], CpSize;
+	    CodePage = -1;
+	    if (!DosQueryCp(sizeof(CpList), CpList, &CpSize))
+		CodePage = *CpList;
 	}
 #endif
     }
-#elif defined(TERMIOS) && defined(TIOCGWINSZ)
+#elif defined(HAVE_TERMIOS_H) && defined(TIOCGWINSZ)
     struct winsize wins;
 
     i = ioctl(tty, TIOCGWINSZ, &wins);
@@ -512,16 +781,10 @@ setlinescols(void)
 	LINES = wins.ws_row;
 	COLS = wins.ws_col;
     }
-#endif				/* defined(TERMIOS) && defined(TIOCGWINSZ) 
-				 * 
-				 */
-    if (LINES <= 0 &&
-	(p = getenv("LINES")) != NULL &&
-	(i = atoi(p)) >= 0)
+#endif				/* defined(HAVE-TERMIOS_H) && defined(TIOCGWINSZ) */
+    if (LINES <= 0 && (p = getenv("LINES")) != NULL && (i = atoi(p)) >= 0)
 	LINES = i;
-    if (COLS <= 0 &&
-	(p = getenv("COLUMNS")) != NULL &&
-	(i = atoi(p)) >= 0)
+    if (COLS <= 0 && (p = getenv("COLUMNS")) != NULL && (i = atoi(p)) >= 0)
 	COLS = i;
     if (LINES <= 0)
 	LINES = tgetnum("li");	/* number of line */
@@ -531,6 +794,9 @@ setlinescols(void)
 	COLS = MAX_COLUMN;
     if (LINES > MAX_LINE)
 	LINES = MAX_LINE;
+#if defined(CYGWIN) && LANG == JA
+    LASTLINE = LINES - (isWinConsole ? 2 : 1);
+#endif				/* defined(CYGWIN) && LANG == JA */
 }
 
 void
@@ -601,7 +867,7 @@ write1(char c)
 	    break;
 	case C_WCHAR1:
 	    wmode = C_WCHAR2;
-	    putchars((unsigned char) wbuf, (unsigned char) c, ttyf);
+	    putchars((unsigned char)wbuf, (unsigned char)c, ttyf);
 	    break;
 	}
     }
@@ -661,7 +927,7 @@ switch_wchar(FILE * f)
 void
 putchars(unsigned char c1, unsigned char c2, FILE * f)
 {
-    Str s;
+    Str s = NULL;
     char *p;
 
     switch (DisplayCode) {
@@ -682,7 +948,7 @@ putchars(unsigned char c1, unsigned char c2, FILE * f)
 	put_sjis(s, c1 & 0x7f, c2 & 0x7f);
 	break;
     }
-    if (! s->length) {
+    if (!s->length) {
 	putc('?', f);
 	putc('?', f);
 	return;
@@ -701,11 +967,11 @@ move(int line, int column)
 	CurColumn = column;
 }
 
-#ifdef BG_COLOR
+#ifdef USE_BG_COLOR
 #define M_SPACE (S_SCREENPROP|S_COLORED|S_BCOLORED|S_GRAPHICS)
-#else				/* not BG_COLOR */
+#else				/* not USE_BG_COLOR */
 #define M_SPACE (S_SCREENPROP|S_COLORED|S_GRAPHICS)
-#endif				/* not BG_COLOR */
+#endif				/* not USE_BG_COLOR */
 
 static int
 need_redraw(char c1, l_prop pr1, char c2, l_prop pr2)
@@ -782,10 +1048,12 @@ addch(char c)
 	    touch_column(CurColumn + 1);
 	    if (pr[CurColumn + 1] & S_EOL) {
 		p[CurColumn + 1] = ' ';
-		SETPROP(pr[CurColumn + 1], (pr[CurColumn + 1] & M_CEOL) | C_ASCII);
+		SETPROP(pr[CurColumn + 1],
+			(pr[CurColumn + 1] & M_CEOL) | C_ASCII);
 	    }
 #ifdef JP_CHARSET
-	    else if (CHMODE(pr[CurColumn + 1]) == C_WCHAR1 && CurColumn < COLS - 2)
+	    else if (CHMODE(pr[CurColumn + 1]) == C_WCHAR1
+		     && CurColumn < COLS - 2)
 		touch_column(CurColumn + 2);
 #endif				/* JP_CHARSET */
 	}
@@ -795,7 +1063,8 @@ addch(char c)
     if (CurColumn >= 1 && CHMODE(pr[CurColumn - 1]) == C_WCHAR1 &&
 	CHMODE(CurrentMode) != C_WCHAR2) {
 	p[CurColumn - 1] = ' ';
-	SETPROP(pr[CurColumn - 1], (pr[CurColumn - 1] & ~C_WHICHCHAR) | C_ASCII);
+	SETPROP(pr[CurColumn - 1],
+		(pr[CurColumn - 1] & ~C_WHICHCHAR) | C_ASCII);
 	touch_line();
 	touch_column(CurColumn - 1);
     }
@@ -803,7 +1072,8 @@ addch(char c)
     if (CurColumn < COLS - 1 && CHMODE(pr[CurColumn + 1]) == C_WCHAR2 &&
 	CHMODE(CurrentMode) != C_WCHAR1) {
 	p[CurColumn + 1] = ' ';
-	SETPROP(pr[CurColumn + 1], (pr[CurColumn + 1] & ~C_WHICHCHAR) | C_ASCII);
+	SETPROP(pr[CurColumn + 1],
+		(pr[CurColumn + 1] & ~C_WHICHCHAR) | C_ASCII);
 	touch_line();
 	touch_column(CurColumn + 1);
     }
@@ -857,8 +1127,7 @@ addch(char c)
     else if (c == '\b' && CurColumn > 0) {	/* Backspace */
 	CurColumn--;
 #ifdef JP_CHARSET
-	if (CurColumn > 0 &&
-	    CHMODE(pr[CurColumn]) == C_WCHAR2)
+	if (CurColumn > 0 && CHMODE(pr[CurColumn]) == C_WCHAR2)
 	    CurColumn--;
 #endif				/* JP_CHARSET */
     }
@@ -977,7 +1246,7 @@ color_seq(int colmode)
     return seqbuf;
 }
 
-#ifdef BG_COLOR
+#ifdef USE_BG_COLOR
 void
 setbcolor(int color)
 {
@@ -993,16 +1262,16 @@ bcolor_seq(int colmode)
     sprintf(seqbuf, "\033[%dm", ((colmode >> 12) & 7) + 40);
     return seqbuf;
 }
-#endif				/* BG_COLOR */
+#endif				/* USE_BG_COLOR */
 
 #define RF_NEED_TO_MOVE    0
 #define RF_CR_OK           1
 #define RF_NONEED_TO_MOVE  2
-#ifdef BG_COLOR
+#ifdef USE_BG_COLOR
 #define M_MEND (S_STANDOUT|S_UNDERLINE|S_BOLD|S_COLORED|S_BCOLORED|S_GRAPHICS)
-#else				/* not BG_COLOR */
+#else				/* not USE_BG_COLOR */
 #define M_MEND (S_STANDOUT|S_UNDERLINE|S_BOLD|S_COLORED|S_GRAPHICS)
-#endif				/* not BG_COLOR */
+#endif				/* not USE_BG_COLOR */
 void
 refresh(void)
 {
@@ -1012,9 +1281,9 @@ refresh(void)
     char *pc;
     l_prop *pr, mode = 0;
     l_prop color = COL_FTERM;
-#ifdef BG_COLOR
+#ifdef USE_BG_COLOR
     l_prop bcolor = COL_BTERM;
-#endif				/* BG_COLOR */
+#endif				/* USE_BG_COLOR */
     short *dirty;
 
     for (line = 0; line <= LASTLINE; line++) {
@@ -1083,34 +1352,35 @@ refresh(void)
 		 * avoid the scroll, I prohibit to draw character on
 		 * (COLS-1,LINES-1).
 		 */
-#if !defined( BG_COLOR ) || defined( CYGWIN )
-#ifdef CYGWIN
+#if !defined(USE_BG_COLOR) || defined(__CYGWIN__)
+#if defined(__CYGWIN__) && LANG == JA
 		if (isWinConsole)
-#endif
-		if (line == LINES - 1 && col == COLS - 1)
-		    break;
-#endif				/* not BG_COLOR */
+#endif				/* defined(__CYGWIN__) && LANG == JA */
+		    if (line == LINES - 1 && col == COLS - 1)
+			break;
+#endif				/* !defined(USE_BG_COLOR) || defined(__CYGWIN__) */
 		if ((!(pr[col] & S_STANDOUT) && (mode & S_STANDOUT)) ||
 		    (!(pr[col] & S_UNDERLINE) && (mode & S_UNDERLINE)) ||
 		    (!(pr[col] & S_BOLD) && (mode & S_BOLD)) ||
 		    (!(pr[col] & S_COLORED) && (mode & S_COLORED))
-#ifdef BG_COLOR
+#ifdef USE_BG_COLOR
 		    || (!(pr[col] & S_BCOLORED) && (mode & S_BCOLORED))
-#endif				/* BG_COLOR */
+#endif				/* USE_BG_COLOR */
 		    || (!(pr[col] & S_GRAPHICS) && (mode & S_GRAPHICS))) {
-            if ((mode & S_COLORED)
-#ifdef BG_COLOR
-              || (mode & S_BCOLORED)
-#endif				/* BG_COLOR */
+		    if ((mode & S_COLORED)
+#ifdef USE_BG_COLOR
+			|| (mode & S_BCOLORED)
+#endif				/* USE_BG_COLOR */
 			)
 			writestr(T_op);
-            if (mode & S_GRAPHICS)
+		    if (mode & S_GRAPHICS)
 			writestr(T_ae);
 		    writestr(T_me);
 		    mode &= ~M_MEND;
 		}
 		if ((*dirty & L_NEED_CE && col >= ScreenImage[line]->eol) ?
-		    need_redraw(pc[col], pr[col], ' ', 0) : (pr[col] & S_DIRTY)) {
+		    need_redraw(pc[col], pr[col], ' ',
+				0) : (pr[col] & S_DIRTY)) {
 		    if (pcol == col - 1)
 			writestr(T_nd);
 		    else if (pcol != col)
@@ -1128,18 +1398,19 @@ refresh(void)
 			writestr(T_md);
 			mode |= S_BOLD;
 		    }
-            if ((pr[col] & S_COLORED) && (pr[col] ^ mode) & COL_FCOLOR) {
-               color = (pr[col] & COL_FCOLOR);
-               mode = ((mode & ~COL_FCOLOR) | color);
-               writestr(color_seq(color));
-            }
-#ifdef BG_COLOR
-            if ((pr[col] & S_BCOLORED) && (pr[col] ^ mode) & COL_BCOLOR) {
-               bcolor = (pr[col] & COL_BCOLOR);
-               mode = ((mode & ~COL_BCOLOR) | bcolor);
-               writestr(bcolor_seq(bcolor));
-            }
-#endif                /* BG_COLOR */
+		    if ((pr[col] & S_COLORED) && (pr[col] ^ mode) & COL_FCOLOR) {
+			color = (pr[col] & COL_FCOLOR);
+			mode = ((mode & ~COL_FCOLOR) | color);
+			writestr(color_seq(color));
+		    }
+#ifdef USE_BG_COLOR
+		    if ((pr[col] & S_BCOLORED)
+			&& (pr[col] ^ mode) & COL_BCOLOR) {
+			bcolor = (pr[col] & COL_BCOLOR);
+			mode = ((mode & ~COL_BCOLOR) | bcolor);
+			writestr(bcolor_seq(bcolor));
+		    }
+#endif				/* USE_BG_COLOR */
 		    if ((pr[col] & S_GRAPHICS) && !(mode & S_GRAPHICS)) {
 			if (!graph_enabled) {
 			    graph_enabled = 1;
@@ -1148,7 +1419,8 @@ refresh(void)
 			writestr(T_as);
 			mode |= S_GRAPHICS;
 		    }
-		    write1((pr[col] & S_GRAPHICS) ? graphchar(pc[col]) : pc[col]);
+		    write1((pr[col] & S_GRAPHICS) ? graphchar(pc[col]) :
+			   pc[col]);
 		    pcol = col + 1;
 		}
 	    }
@@ -1160,9 +1432,9 @@ refresh(void)
 	*dirty &= ~(L_NEED_CE | L_CLRTOEOL);
 	if (mode & M_MEND) {
 	    if (mode & (S_COLORED
-#ifdef BG_COLOR
+#ifdef USE_BG_COLOR
 			| S_BCOLORED
-#endif				/* BG_COLOR */
+#endif				/* USE_BG_COLOR */
 		))
 		writestr(T_op);
 	    if (mode & S_GRAPHICS)
@@ -1305,6 +1577,7 @@ need_clrtoeol(void)
 }
 #endif				/* 0 */
 
+/* XXX: conflicts with curses's clrtoeol(3) ? */
 void
 clrtoeol(void)
 {				/* Clear to the end of line */
@@ -1325,7 +1598,7 @@ clrtoeol(void)
     }
 }
 
-#ifdef BG_COLOR
+#ifdef USE_BG_COLOR
 void
 clrtoeol_with_bcolor(void)
 {
@@ -1351,14 +1624,14 @@ clrtoeolx(void)
 {
     clrtoeol_with_bcolor();
 }
-#else				/* not BG_COLOR */
+#else				/* not USE_BG_COLOR */
 
 void
 clrtoeolx(void)
 {
     clrtoeol();
 }
-#endif				/* not BG_COLOR */
+#endif				/* not USE_BG_COLOR */
 
 void
 clrtobot_eol(void (*clrtoeol) ())
@@ -1466,38 +1739,38 @@ addnstr_sup(char *s, int n)
 
 void
 crmode(void)
-#ifndef SGTTY
+#ifndef HAVE_SGTTY_H
 {
     ttymode_reset(ICANON, IXON);
     ttymode_set(ISIG, 0);
-#ifdef TERMIOS
+#ifdef HAVE_TERMIOS_H
     set_cc(VMIN, 1);
-#else				/* not TERMIOS */
+#else				/* not HAVE_TERMIOS_H */
     set_cc(VEOF, 1);
-#endif				/* not TERMIOS */
+#endif				/* not HAVE_TERMIOS_H */
 }
-#else				/* SGTTY */
+#else				/* HAVE_SGTTY_H */
 {
     ttymode_set(CBREAK, 0);
 }
-#endif				/* SGTTY */
+#endif				/* HAVE_SGTTY_H */
 
 void
 nocrmode(void)
-#ifndef SGTTY
+#ifndef HAVE_SGTTY_H
 {
     ttymode_set(ICANON, 0);
-#ifdef TERMIOS
+#ifdef HAVE_TERMIOS_H
     set_cc(VMIN, 4);
-#else				/* not TERMIOS */
+#else				/* not HAVE_TERMIOS_H */
     set_cc(VEOF, 4);
-#endif				/* not TERMIOS */
+#endif				/* not HAVE_TERMIOS_H */
 }
-#else				/* SGTTY */
+#else				/* HAVE_SGTTY_H */
 {
     ttymode_reset(CBREAK, 0);
 }
-#endif				/* SGTTY */
+#endif				/* HAVE_SGTTY_H */
 
 void
 term_echo(void)
@@ -1513,7 +1786,7 @@ term_noecho(void)
 
 void
 term_raw(void)
-#ifndef SGTTY
+#ifndef HAVE_SGTTY_H
 #ifdef IEXTEN
 #define TTY_MODE ISIG|ICANON|ECHO|IEXTEN
 #else				/* not IEXTEN */
@@ -1521,41 +1794,41 @@ term_raw(void)
 #endif				/* not IEXTEN */
 {
     ttymode_reset(TTY_MODE, IXON | IXOFF);
-#ifdef TERMIOS
+#ifdef HAVE_TERMIOS_H
     set_cc(VMIN, 1);
-#else				/* not TERMIOS */
+#else				/* not HAVE_TERMIOS_H */
     set_cc(VEOF, 1);
-#endif				/* not TERMIOS */
+#endif				/* not HAVE_TERMIOS_H */
 }
-#else				/* SGTTY */
+#else				/* HAVE_SGTTY_H */
 {
     ttymode_set(RAW, 0);
 }
-#endif				/* SGTTY */
+#endif				/* HAVE_SGTTY_H */
 
 void
 term_cooked(void)
-#ifndef SGTTY
+#ifndef HAVE_SGTTY_H
 {
 #ifdef __EMX__
     /* On XFree86/OS2, some scrambled characters
      * will appear when asserting IEXTEN flag.
      */
-    ttymode_set((TTY_MODE)&~IEXTEN,0);
+    ttymode_set((TTY_MODE) & ~IEXTEN, 0);
 #else
     ttymode_set(TTY_MODE, 0);
 #endif
-#ifdef TERMIOS
+#ifdef HAVE_TERMIOS_H
     set_cc(VMIN, 4);
-#else				/* not TERMIOS */
+#else				/* not HAVE_TERMIOS_H */
     set_cc(VEOF, 4);
-#endif				/* not TERMIOS */
+#endif				/* not HAVE_TERMIOS_H */
 }
-#else				/* SGTTY */
+#else				/* HAVE_SGTTY_H */
 {
     ttymode_reset(RAW, 0);
 }
-#endif				/* SGTTY */
+#endif				/* HAVE_SGTTY_H */
 
 void
 term_cbreak(void)
@@ -1569,16 +1842,23 @@ getch(void)
 {
     char c;
 
-    while (read(tty, &c, 1) < (int)1) {
-        if (errno == EINTR || errno == EAGAIN) continue;
-        /* error happend on read(2) */
-        quitfm();
-        break;  /* unreachable */
+    while (
+#ifdef __CYGWIN__
+	      read_win32_console(&c, 1)
+#else
+	      read(tty, &c, 1)
+#endif
+	      < (int)1) {
+	if (errno == EINTR || errno == EAGAIN)
+	    continue;
+	/* error happend on read(2) */
+	quitfm();
+	break;			/* unreachable */
     }
     return c;
 }
 
-#ifdef MOUSE
+#ifdef USE_MOUSE
 #ifdef USE_GPM
 char
 wgetch(void)
@@ -1623,7 +1903,7 @@ sysm_getch()
 int
 do_getch()
 {
-    if (is_xterm || ! sysm_handler)
+    if (is_xterm || !sysm_handler)
 	return getch();
     else
 	return sysm_getch();
@@ -1648,7 +1928,7 @@ sysmouse(SIGNAL_ARG)
     ioctl(tty, CONS_MOUSECTL, &mi);
 }
 #endif				/* USE_SYSMOUSE */
-#endif				/* MOUSE */
+#endif				/* USE_MOUSE */
 
 void
 bell(void)
@@ -1664,8 +1944,16 @@ skip_escseq(void)
     c = getch();
     if (c == '[' || c == 'O') {
 	c = getch();
-	while (IS_DIGIT(c))
-	    c = getch();
+#ifdef USE_MOUSE
+	if (is_xterm && c == 'M') {
+	    getch();
+	    getch();
+	    getch();
+	}
+	else
+#endif
+	    while (IS_DIGIT(c))
+		c = getch();
     }
 }
 
@@ -1686,7 +1974,13 @@ sleep_till_anykey(int sec, int purge)
     FD_ZERO(&rfd);
     FD_SET(tty, &rfd);
 
-    if (select(tty + 1, &rfd, 0, 0, &tim) > 0 && purge) {
+    if (
+#ifdef __CYGWIN__
+	   select_win32_console(tty + 1, &rfd, 0, 0, &tim)
+#else
+	   select(tty + 1, &rfd, 0, 0, &tim)
+#endif
+	   > 0 && purge) {
 	c = getch();
 	if (c == ESC_CODE)
 	    skip_escseq();
@@ -1698,7 +1992,7 @@ sleep_till_anykey(int sec, int purge)
     }
 }
 
-#ifdef MOUSE
+#ifdef USE_MOUSE
 
 #define XTERM_ON   {fputs("\033[?1001s\033[?1000h",ttyf); flush_tty();}
 #define XTERM_OFF  {fputs("\033[?1000l\033[?1001r",ttyf); flush_tty();}
@@ -1711,20 +2005,37 @@ mouse_init()
 {
     Gpm_Connect conn;
     extern int gpm_process_mouse(Gpm_Event *, void *);
+    int r;
 
     if (mouseActive)
 	return;
+    conn.eventMask = ~0;
+    conn.defaultMask = 0;
+    conn.maxMod = 0;
+    conn.minMod = 0;
+
+    r = Gpm_Open(&conn, 0);
+    if (r == -2) {
+	/*
+	 * If Gpm_Open() success, returns >= 0
+	 * Gpm_Open() returns -2 in case of xterm.
+	 * Gpm_Close() is necessary here. Otherwise,
+	 * xterm is being left in the mode where the mouse clicks are
+	 * passed through to the application.
+	 */
+	Gpm_Close();
+	is_xterm = (NEED_XTERM_ON | NEED_XTERM_OFF);
+    }
+    else if (r >= 0) {
+	gpm_handler = gpm_process_mouse;
+	is_xterm = 0;
+    }
     if (is_xterm) {
 	XTERM_ON;
     }
-    else {
-	conn.eventMask = ~0;
-	conn.defaultMask = 0;
-	conn.maxMod = 0;
-	conn.minMod = 0;
-	Gpm_Open(&conn, 0);	/* don't care even if it fails */
-	gpm_handler = gpm_process_mouse;
-    }
+#ifdef SIGWINCH
+    signal(SIGWINCH, resize_hook);
+#endif
     mouseActive = 1;
 }
 
@@ -1773,7 +2084,7 @@ mouse_init()
 	mi.operation = MOUSE_MODE;
 	mi.u.mode.mode = 0;
 	mi.u.mode.signal = SIGUSR2;
-       sysm_handler = NULL;
+	sysm_handler = NULL;
 	if (ioctl(tty, CONS_MOUSECTL, &mi) != -1) {
 	    signal(SIGUSR2, sysmouse);
 	    mi.operation = MOUSE_SHOW;
@@ -1810,7 +2121,7 @@ mouse_init()
 {
     if (mouseActive)
 	return;
-    if (is_xterm) {
+    if (is_xterm & NEED_XTERM_ON) {
 	XTERM_ON;
     }
     mouseActive = 1;
@@ -1821,7 +2132,7 @@ mouse_end()
 {
     if (mouseActive == 0)
 	return;
-    if (is_xterm) {
+    if (is_xterm & NEED_XTERM_OFF) {
 	XTERM_OFF;
     }
     mouseActive = 0;
@@ -1844,7 +2155,7 @@ mouse_inactive()
 	mouse_end();
 }
 
-#endif				/* MOUSE */
+#endif				/* USE_MOUSE */
 
 void
 flush_tty()
